/* 
 *            Copyright 2009-2016 The VOTCA Development Team
 *                       (http://www.votca.org)
 *
 *      Licensed under the Apache License, Version 2.0 (the "License")
 *
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

// Overload of uBLAS prod function with MKL/GSL implementations


#include <votca/xtp/threecenters.h>


using namespace votca::tools;

namespace votca {
    namespace xtp {
        namespace ub = boost::numeric::ublas;

 
        
        /*
         * Calculate 3-center overlap integrals 
         *    S_{abc} = int{ phi_a^DFT phi_b^GW phi_c^DFT d3r }
         * for a given set of a b c as in 
         *    Obara, Saika, J. Chem. Phys. 84, 3963 (1986)
         * section II.B for cartesian Gaussians, then transforming
         * to spherical (angular momentum) Gaussians ("complete" shells
         * from S to Lmax, and finally cutting out those angular momentum 
         * components actually present in shell-shell-shell combination.
         * Currently supported for 
         *      S,P,D   functions in DFT basis and 
         *      S,P,D,F functions in GW  basis
         * 
         */
        void TCrawMatrix::FillThreeCenterOLBlock_g(ma_type &S,vec &gma, vec &gmc,int &_lmax_alpha, int &_lmax_gamma,double & fak) {
	  //bool TCrawMatrix::FillThreeCenterOLBlock(ub::matrix<float>& _subvector, AOShell* _shell_gw, AOShell* _shell_alpha, AOShell* _shell_gamma) {
            
            double fak2 = 2.0 * fak;
            double fak3 = 3.0 * fak;
            double fak4=  4.0 * fak;
 
            double gma0 = gma.getX();
            double gmc0 = gmc.getX();
            double gma1 = gma.getY();
            double gmc1 = gmc.getY();
            double gma2 = gma.getZ();
            double gmc2 = gmc.getZ();
                    

     S[1][21][1] = gmc0*S[1][11][1]+     fak3* S[1][8][1];
     S[1][24][1] = gmc1*S[1][11][1];
     S[1][26][1] = gmc2*S[1][11][1];
     S[1][25][1] = gmc0*S[1][12][1];
     S[1][22][1] = gmc1*S[1][12][1]+     fak3* S[1][9][1];
     S[1][28][1] = gmc2*S[1][12][1];
     S[1][27][1] = gmc0*S[1][13][1];
     S[1][29][1] = gmc1*S[1][13][1];
     S[1][23][1] = gmc2*S[1][13][1]+     fak3* S[1][10][1];
     S[1][32][1] = gmc1*S[1][14][1]+     fak * S[1][8][1];
     S[1][33][1] = gmc2*S[1][14][1];
     S[1][34][1] = gmc2*S[1][15][1];
     S[1][30][1] = gmc2*S[1][16][1]+     fak * S[1][8][1];
     S[1][35][1] = gmc1*S[1][17][1];
     S[1][31][1] = gmc2*S[1][18][1]+     fak * S[1][9][1];
  
  if (   _lmax_gamma>=1 ) {
     S[1][21][2] = gmc0*S[1][11][2]+     fak * (3e0*S[1][8][2] + S[1][11][1] );
     S[1][24][2] = gmc1*S[1][11][2];
     S[1][26][2] = gmc2*S[1][11][2];
     S[1][21][3] = gmc0*S[1][11][3]+     fak3* S[1][8][3];
     S[1][24][3] = gmc1*S[1][11][3]+     fak * S[1][11][1];
     S[1][26][3] = gmc2*S[1][11][3];
     S[1][21][4] = gmc0*S[1][11][4]+     fak3* S[1][8][4];
     S[1][24][4] = gmc1*S[1][11][4];
     S[1][26][4] = gmc2*S[1][11][4]+     fak * S[1][11][1];
     S[1][25][2] = gmc0*S[1][12][2]+     fak * S[1][12][1];
     S[1][22][2] = gmc1*S[1][12][2]+     fak3* S[1][9][2];
     S[1][28][2] = gmc2*S[1][12][2];
     S[1][25][3] = gmc0*S[1][12][3];
     S[1][22][3] = gmc1*S[1][12][3]+     fak * (3e0*S[1][9][3] + S[1][12][1] );
     S[1][28][3] = gmc2*S[1][12][3];
     S[1][25][4] = gmc0*S[1][12][4];
     S[1][22][4] = gmc1*S[1][12][4]+     fak3* S[1][9][4];
     S[1][28][4] = gmc2*S[1][12][4]+     fak * S[1][12][1];
     S[1][27][2] = gmc0*S[1][13][2]+     fak * S[1][13][1];
     S[1][29][2] = gmc1*S[1][13][2];
     S[1][23][2] = gmc2*S[1][13][2]+     fak3* S[1][10][2];
     S[1][27][3] = gmc0*S[1][13][3];
     S[1][29][3] = gmc1*S[1][13][3]+     fak * S[1][13][1];
     S[1][23][3] = gmc2*S[1][13][3]+     fak3* S[1][10][3];
     S[1][27][4] = gmc0*S[1][13][4];
     S[1][29][4] = gmc1*S[1][13][4];
     S[1][23][4] = gmc2*S[1][13][4]+     fak * (3e0*S[1][10][4] + S[1][13][1] );
     S[1][32][2] = gmc1*S[1][14][2]+     fak * S[1][8][2];
     S[1][33][2] = gmc2*S[1][14][2];
     S[1][32][3] = gmc1*S[1][14][3]+     fak * (S[1][8][3] + S[1][14][1] );
     S[1][33][3] = gmc2*S[1][14][3];
     S[1][32][4] = gmc1*S[1][14][4]+     fak * S[1][8][4];
     S[1][33][4] = gmc2*S[1][14][4]+     fak * S[1][14][1];
     S[1][34][2] = gmc2*S[1][15][2];
     S[1][34][3] = gmc2*S[1][15][3];
     S[1][34][4] = gmc2*S[1][15][4]+     fak * S[1][15][1];
     S[1][30][2] = gmc2*S[1][16][2]+     fak * S[1][8][2];
     S[1][30][3] = gmc2*S[1][16][3]+     fak * S[1][8][3];
     S[1][30][4] = gmc2*S[1][16][4]+     fak * (S[1][8][4] + S[1][16][1] );
     S[1][35][2] = gmc1*S[1][17][2];
     S[1][35][3] = gmc1*S[1][17][3]+     fak * S[1][17][1];
     S[1][35][4] = gmc1*S[1][17][4];
     S[1][31][2] = gmc2*S[1][18][2]+     fak * S[1][9][2];
     S[1][31][3] = gmc2*S[1][18][3]+     fak * S[1][9][3];
     S[1][31][4] = gmc2*S[1][18][4]+     fak * (S[1][9][4] + S[1][18][1] );
  }
  if ( _lmax_alpha>=1  ) {
     S[2][21][1] = gma0*S[1][21][1]+     fak4* S[1][11][1];
     S[3][21][1] = gma1*S[1][21][1];
     S[4][21][1] = gma2*S[1][21][1];
     S[2][22][1] = gma0*S[1][22][1];
     S[3][22][1] = gma1*S[1][22][1]+     fak4* S[1][12][1];
     S[4][22][1] = gma2*S[1][22][1];
     S[2][23][1] = gma0*S[1][23][1];
     S[3][23][1] = gma1*S[1][23][1];
     S[4][23][1] = gma2*S[1][23][1]+     fak4* S[1][13][1];
     S[2][24][1] = gma0*S[1][24][1]+     fak3* S[1][14][1];
     S[3][24][1] = gma1*S[1][24][1]+     fak * S[1][11][1];
     S[4][24][1] = gma2*S[1][24][1];
     S[2][25][1] = gma0*S[1][25][1]+     fak * S[1][12][1];
     S[3][25][1] = gma1*S[1][25][1]+     fak3* S[1][15][1];
     S[4][25][1] = gma2*S[1][25][1];
     S[2][26][1] = gma0*S[1][26][1]+     fak3* S[1][16][1];
     S[3][26][1] = gma1*S[1][26][1];
     S[4][26][1] = gma2*S[1][26][1]+     fak * S[1][11][1];
     S[2][27][1] = gma0*S[1][27][1]+     fak * S[1][13][1];
     S[3][27][1] = gma1*S[1][27][1];
     S[4][27][1] = gma2*S[1][27][1]+     fak3* S[1][17][1];
     S[2][28][1] = gma0*S[1][28][1];
     S[3][28][1] = gma1*S[1][28][1]+     fak3* S[1][18][1];
     S[4][28][1] = gma2*S[1][28][1]+     fak * S[1][12][1];
     S[2][29][1] = gma0*S[1][29][1];
     S[3][29][1] = gma1*S[1][29][1]+     fak * S[1][13][1];
     S[4][29][1] = gma2*S[1][29][1]+     fak3* S[1][19][1];
     S[2][30][1] = gma0*S[1][30][1]+     fak2* S[1][17][1];
     S[3][30][1] = gma1*S[1][30][1];
     S[4][30][1] = gma2*S[1][30][1]+     fak2* S[1][16][1];
     S[2][31][1] = gma0*S[1][31][1];
     S[3][31][1] = gma1*S[1][31][1]+     fak2* S[1][19][1];
     S[4][31][1] = gma2*S[1][31][1]+     fak2* S[1][18][1];
     S[2][32][1] = gma0*S[1][32][1]+     fak2* S[1][15][1];
     S[3][32][1] = gma1*S[1][32][1]+     fak2* S[1][14][1];
     S[4][32][1] = gma2*S[1][32][1];
     S[2][33][1] = gma0*S[1][33][1]+     fak2* S[1][20][1];
     S[3][33][1] = gma1*S[1][33][1]+     fak * S[1][16][1];
     S[4][33][1] = gma2*S[1][33][1]+     fak * S[1][14][1];
     S[2][34][1] = gma0*S[1][34][1]+     fak * S[1][18][1];
     S[3][34][1] = gma1*S[1][34][1]+     fak2* S[1][20][1];
     S[4][34][1] = gma2*S[1][34][1]+     fak * S[1][15][1];
     S[2][35][1] = gma0*S[1][35][1]+     fak * S[1][19][1];
     S[3][35][1] = gma1*S[1][35][1]+     fak * S[1][17][1];
     S[4][35][1] = gma2*S[1][35][1]+     fak2* S[1][20][1];
  }
  if (   _lmax_gamma>=2 ) {
     S[1][21][5] = gmc0*S[1][11][5]+     fak * (3.e0*S[1][8][5] + S[1][11][3] );
     S[1][24][5] = gmc1*S[1][11][5]+     fak * S[1][11][2];
     S[1][26][5] = gmc2*S[1][11][5];
     S[1][21][6] = gmc0*S[1][11][6]+     fak * (3.e0*S[1][8][6] + S[1][11][4] );
     S[1][24][6] = gmc1*S[1][11][6];
     S[1][26][6] = gmc2*S[1][11][6]+     fak * S[1][11][2];
     S[1][21][7] = gmc0*S[1][11][7]+     fak3* S[1][8][7];
     S[1][24][7] = gmc1*S[1][11][7]+     fak * S[1][11][4];
     S[1][26][7] = gmc2*S[1][11][7]+     fak * S[1][11][3];
     S[1][21][8] = gmc0*S[1][11][8]+     fak * (3.e0*S[1][8][8] + 2.e0*S[1][11][2] );
     S[1][24][8] = gmc1*S[1][11][8];
     S[1][26][8] = gmc2*S[1][11][8];
     S[1][21][9] = gmc0*S[1][11][9]+     fak3* S[1][8][9];
     S[1][24][9] = gmc1*S[1][11][9]+     fak2* S[1][11][3];
     S[1][26][9] = gmc2*S[1][11][9];
     S[1][21][10] = gmc0*S[1][11][10]+     fak3* S[1][8][10];
     S[1][24][10] = gmc1*S[1][11][10];
     S[1][26][10] = gmc2*S[1][11][10]+     fak2* S[1][11][4];
     S[1][25][5] = gmc0*S[1][12][5]+     fak * S[1][12][3];
     S[1][22][5] = gmc1*S[1][12][5]+     fak * (3.e0*S[1][9][5] + S[1][12][2] );
     S[1][28][5] = gmc2*S[1][12][5];
     S[1][25][6] = gmc0*S[1][12][6]+     fak * S[1][12][4];
     S[1][22][6] = gmc1*S[1][12][6]+     fak3* S[1][9][6];
     S[1][28][6] = gmc2*S[1][12][6]+     fak * S[1][12][2];
     S[1][25][7] = gmc0*S[1][12][7];
     S[1][22][7] = gmc1*S[1][12][7]+     fak * (3.e0*S[1][9][7] + S[1][12][4] );
     S[1][28][7] = gmc2*S[1][12][7]+     fak * S[1][12][3];
     S[1][25][8] = gmc0*S[1][12][8]+     fak2* S[1][12][2];
     S[1][22][8] = gmc1*S[1][12][8]+     fak3* S[1][9][8];
     S[1][28][8] = gmc2*S[1][12][8];
     S[1][25][9] = gmc0*S[1][12][9];
     S[1][22][9] = gmc1*S[1][12][9]+     fak * (3.e0*S[1][9][9] + 2.e0*S[1][12][3] );
     S[1][28][9] = gmc2*S[1][12][9];
     S[1][25][10] = gmc0*S[1][12][10];
     S[1][22][10] = gmc1*S[1][12][10]+     fak3* S[1][9][10];
     S[1][28][10] = gmc2*S[1][12][10]+     fak2* S[1][12][4];
     S[1][27][5] = gmc0*S[1][13][5]+     fak * S[1][13][3];
     S[1][29][5] = gmc1*S[1][13][5]+     fak * S[1][13][2];
     S[1][23][5] = gmc2*S[1][13][5]+     fak3* S[1][10][5];
     S[1][27][6] = gmc0*S[1][13][6]+     fak * S[1][13][4];
     S[1][29][6] = gmc1*S[1][13][6];
     S[1][23][6] = gmc2*S[1][13][6]+     fak * (3.e0*S[1][10][6] + S[1][13][2] );
     S[1][27][7] = gmc0*S[1][13][7];
     S[1][29][7] = gmc1*S[1][13][7]+     fak * S[1][13][4];
     S[1][23][7] = gmc2*S[1][13][7]+     fak * (3.e0*S[1][10][7] + S[1][13][3] );
     S[1][27][8] = gmc0*S[1][13][8]+     fak2* S[1][13][2];
     S[1][29][8] = gmc1*S[1][13][8];
     S[1][23][8] = gmc2*S[1][13][8]+     fak3* S[1][10][8];
     S[1][27][9] = gmc0*S[1][13][9];
     S[1][29][9] = gmc1*S[1][13][9]+     fak2* S[1][13][3];
     S[1][23][9] = gmc2*S[1][13][9]+     fak3* S[1][10][9];
     S[1][27][10] = gmc0*S[1][13][10];
     S[1][29][10] = gmc1*S[1][13][10];
     S[1][23][10] = gmc2*S[1][13][10]+     fak * (3.e0*S[1][10][10] + 2.e0*S[1][13][4] );
     S[1][32][5] = gmc1*S[1][14][5]+     fak * (S[1][8][5] + S[1][14][2] );
     S[1][33][5] = gmc2*S[1][14][5];
     S[1][32][6] = gmc1*S[1][14][6]+     fak * S[1][8][6];
     S[1][33][6] = gmc2*S[1][14][6]+     fak * S[1][14][2];
     S[1][32][7] = gmc1*S[1][14][7]+     fak * (S[1][8][7] + S[1][14][4] );
     S[1][33][7] = gmc2*S[1][14][7]+     fak * S[1][14][3];
     S[1][32][8] = gmc1*S[1][14][8]+     fak * S[1][8][8];
     S[1][33][8] = gmc2*S[1][14][8];
     S[1][32][9] = gmc1*S[1][14][9]+     fak * (S[1][8][9] + 2.e0*S[1][14][3] );
     S[1][33][9] = gmc2*S[1][14][9];
     S[1][32][10] = gmc1*S[1][14][10]+     fak * S[1][8][10];
     S[1][33][10] = gmc2*S[1][14][10]+     fak2* S[1][14][4];
     S[1][34][5] = gmc2*S[1][15][5];
     S[1][34][6] = gmc2*S[1][15][6]+     fak * S[1][15][2];
     S[1][34][7] = gmc2*S[1][15][7]+     fak * S[1][15][3];
     S[1][34][8] = gmc2*S[1][15][8];
     S[1][34][9] = gmc2*S[1][15][9];
     S[1][34][10] = gmc2*S[1][15][10]+     fak2* S[1][15][4];
     S[1][30][5] = gmc2*S[1][16][5]+     fak * S[1][8][5];
     S[1][30][6] = gmc2*S[1][16][6]+     fak * (S[1][8][6] + S[1][16][2] );
     S[1][30][7] = gmc2*S[1][16][7]+     fak * (S[1][8][7] + S[1][16][3] );
     S[1][30][8] = gmc2*S[1][16][8]+     fak * S[1][8][8];
     S[1][30][9] = gmc2*S[1][16][9]+     fak * S[1][8][9];
     S[1][30][10] = gmc2*S[1][16][10]+     fak * (S[1][8][10] + 2.e0*S[1][16][4] );
     S[1][35][5] = gmc1*S[1][17][5]+     fak * S[1][17][2];
     S[1][35][6] = gmc1*S[1][17][6];
     S[1][35][7] = gmc1*S[1][17][7]+     fak * S[1][17][4];
     S[1][35][8] = gmc1*S[1][17][8];
     S[1][35][9] = gmc1*S[1][17][9]+     fak2* S[1][17][3];
     S[1][35][10] = gmc1*S[1][17][10];
     S[1][31][5] = gmc2*S[1][18][5]+     fak * S[1][9][5];
     S[1][31][6] = gmc2*S[1][18][6]+     fak * (S[1][9][6] + S[1][18][2] );
     S[1][31][7] = gmc2*S[1][18][7]+     fak * (S[1][9][7] + S[1][18][3] );
     S[1][31][8] = gmc2*S[1][18][8]+     fak * S[1][9][8];
     S[1][31][9] = gmc2*S[1][18][9]+     fak * S[1][9][9];
     S[1][31][10] = gmc2*S[1][18][10]+     fak * (S[1][9][10] + 2.e0*S[1][18][4] );
  }
  if ( _lmax_alpha>=1  &&  _lmax_gamma>=1 ) {
     S[2][21][2] = gma0*S[1][21][2]+     fak * (4.e0*S[1][11][2] + S[1][21][1] );
     S[3][21][2] = gma1*S[1][21][2];
     S[4][21][2] = gma2*S[1][21][2];
     S[2][21][3] = gma0*S[1][21][3]+     fak4* S[1][11][3];
     S[3][21][3] = gma1*S[1][21][3]+     fak * S[1][21][1];
     S[4][21][3] = gma2*S[1][21][3];
     S[2][21][4] = gma0*S[1][21][4]+     fak4* S[1][11][4];
     S[3][21][4] = gma1*S[1][21][4];
     S[4][21][4] = gma2*S[1][21][4]+     fak * S[1][21][1];
     S[2][22][2] = gma0*S[1][22][2]+     fak * S[1][22][1];
     S[3][22][2] = gma1*S[1][22][2]+     fak4* S[1][12][2];
     S[4][22][2] = gma2*S[1][22][2];
     S[2][22][3] = gma0*S[1][22][3];
     S[3][22][3] = gma1*S[1][22][3]+     fak * (4.e0*S[1][12][3] + S[1][22][1] );
     S[4][22][3] = gma2*S[1][22][3];
     S[2][22][4] = gma0*S[1][22][4];
     S[3][22][4] = gma1*S[1][22][4]+     fak4* S[1][12][4];
     S[4][22][4] = gma2*S[1][22][4]+     fak * S[1][22][1];
     S[2][23][2] = gma0*S[1][23][2]+     fak * S[1][23][1];
     S[3][23][2] = gma1*S[1][23][2];
     S[4][23][2] = gma2*S[1][23][2]+     fak4* S[1][13][2];
     S[2][23][3] = gma0*S[1][23][3];
     S[3][23][3] = gma1*S[1][23][3]+     fak * S[1][23][1];
     S[4][23][3] = gma2*S[1][23][3]+     fak4* S[1][13][3];
     S[2][23][4] = gma0*S[1][23][4];
     S[3][23][4] = gma1*S[1][23][4];
     S[4][23][4] = gma2*S[1][23][4]+     fak * (4.e0*S[1][13][4] + S[1][23][1] );
     S[2][24][2] = gma0*S[1][24][2]+     fak * (3.e0*S[1][14][2] + S[1][24][1] );
     S[3][24][2] = gma1*S[1][24][2]+     fak * S[1][11][2];
     S[4][24][2] = gma2*S[1][24][2];
     S[2][24][3] = gma0*S[1][24][3]+     fak3* S[1][14][3];
     S[3][24][3] = gma1*S[1][24][3]+     fak * (S[1][11][3] + S[1][24][1] );
     S[4][24][3] = gma2*S[1][24][3];
     S[2][24][4] = gma0*S[1][24][4]+     fak3* S[1][14][4];
     S[3][24][4] = gma1*S[1][24][4]+     fak * S[1][11][4];
     S[4][24][4] = gma2*S[1][24][4]+     fak * S[1][24][1];
     S[2][25][2] = gma0*S[1][25][2]+     fak * (S[1][12][2] + S[1][25][1] );
     S[3][25][2] = gma1*S[1][25][2]+     fak3* S[1][15][2];
     S[4][25][2] = gma2*S[1][25][2];
     S[2][25][3] = gma0*S[1][25][3]+     fak * S[1][12][3];
     S[3][25][3] = gma1*S[1][25][3]+     fak * (3.e0*S[1][15][3] + S[1][25][1] );
     S[4][25][3] = gma2*S[1][25][3];
     S[2][25][4] = gma0*S[1][25][4]+     fak * S[1][12][4];
     S[3][25][4] = gma1*S[1][25][4]+     fak3* S[1][15][4];
     S[4][25][4] = gma2*S[1][25][4]+     fak * S[1][25][1];
     S[2][26][2] = gma0*S[1][26][2]+     fak * (3.e0*S[1][16][2] + S[1][26][1] );
     S[3][26][2] = gma1*S[1][26][2];
     S[4][26][2] = gma2*S[1][26][2]+     fak * S[1][11][2];
     S[2][26][3] = gma0*S[1][26][3]+     fak3* S[1][16][3];
     S[3][26][3] = gma1*S[1][26][3]+     fak * S[1][26][1];
     S[4][26][3] = gma2*S[1][26][3]+     fak * S[1][11][3];
     S[2][26][4] = gma0*S[1][26][4]+     fak3* S[1][16][4];
     S[3][26][4] = gma1*S[1][26][4];
     S[4][26][4] = gma2*S[1][26][4]+     fak * (S[1][11][4] + S[1][26][1] );
     S[2][27][2] = gma0*S[1][27][2]+     fak * (S[1][13][2] + S[1][27][1] );
     S[3][27][2] = gma1*S[1][27][2];
     S[4][27][2] = gma2*S[1][27][2]+     fak3* S[1][17][2];
     S[2][27][3] = gma0*S[1][27][3]+     fak * S[1][13][3];
     S[3][27][3] = gma1*S[1][27][3]+     fak * S[1][27][1];
     S[4][27][3] = gma2*S[1][27][3]+     fak3* S[1][17][3];
     S[2][27][4] = gma0*S[1][27][4]+     fak * S[1][13][4];
     S[3][27][4] = gma1*S[1][27][4];
     S[4][27][4] = gma2*S[1][27][4]+     fak * (3.e0*S[1][17][4] + S[1][27][1] );
     S[2][28][2] = gma0*S[1][28][2]+     fak * S[1][28][1];
     S[3][28][2] = gma1*S[1][28][2]+     fak3* S[1][18][2];
     S[4][28][2] = gma2*S[1][28][2]+     fak * S[1][12][2];
     S[2][28][3] = gma0*S[1][28][3];
     S[3][28][3] = gma1*S[1][28][3]+     fak * (3.e0*S[1][18][3] + S[1][28][1] );
     S[4][28][3] = gma2*S[1][28][3]+     fak * S[1][12][3];
     S[2][28][4] = gma0*S[1][28][4];
     S[3][28][4] = gma1*S[1][28][4]+     fak3* S[1][18][4];
     S[4][28][4] = gma2*S[1][28][4]+     fak * (S[1][12][4] + S[1][28][1] );
     S[2][29][2] = gma0*S[1][29][2]+     fak * S[1][29][1];
     S[3][29][2] = gma1*S[1][29][2]+     fak * S[1][13][2];
     S[4][29][2] = gma2*S[1][29][2]+     fak3* S[1][19][2];
     S[2][29][3] = gma0*S[1][29][3];
     S[3][29][3] = gma1*S[1][29][3]+     fak * (S[1][13][3] + S[1][29][1] );
     S[4][29][3] = gma2*S[1][29][3]+     fak3* S[1][19][3];
     S[2][29][4] = gma0*S[1][29][4];
     S[3][29][4] = gma1*S[1][29][4]+     fak * S[1][13][4];
     S[4][29][4] = gma2*S[1][29][4]+     fak * (3.e0*S[1][19][4] + S[1][29][1] );
     S[2][30][2] = gma0*S[1][30][2]+     fak * (2.e0*S[1][17][2] + S[1][30][1] );
     S[3][30][2] = gma1*S[1][30][2];
     S[4][30][2] = gma2*S[1][30][2]+     fak2* S[1][16][2];
     S[2][30][3] = gma0*S[1][30][3]+     fak2* S[1][17][3];
     S[3][30][3] = gma1*S[1][30][3]+     fak * S[1][30][1];
     S[4][30][3] = gma2*S[1][30][3]+     fak2* S[1][16][3];
     S[2][30][4] = gma0*S[1][30][4]+     fak2* S[1][17][4];
     S[3][30][4] = gma1*S[1][30][4];
     S[4][30][4] = gma2*S[1][30][4]+     fak * (2.e0*S[1][16][4] + S[1][30][1] );
     S[2][31][2] = gma0*S[1][31][2]+     fak * S[1][31][1];
     S[3][31][2] = gma1*S[1][31][2]+     fak2* S[1][19][2];
     S[4][31][2] = gma2*S[1][31][2]+     fak2* S[1][18][2];
     S[2][31][3] = gma0*S[1][31][3];
     S[3][31][3] = gma1*S[1][31][3]+     fak * (2.e0*S[1][19][3] + S[1][31][1] );
     S[4][31][3] = gma2*S[1][31][3]+     fak2* S[1][18][3];
     S[2][31][4] = gma0*S[1][31][4];
     S[3][31][4] = gma1*S[1][31][4]+     fak2* S[1][19][4];
     S[4][31][4] = gma2*S[1][31][4]+     fak * (2.e0*S[1][18][4] + S[1][31][1] );
     S[2][32][2] = gma0*S[1][32][2]+     fak * (2.e0*S[1][15][2] + S[1][32][1] );
     S[3][32][2] = gma1*S[1][32][2]+     fak2* S[1][14][2];
     S[4][32][2] = gma2*S[1][32][2];
     S[2][32][3] = gma0*S[1][32][3]+     fak2* S[1][15][3];
     S[3][32][3] = gma1*S[1][32][3]+     fak * (2.e0*S[1][14][3] + S[1][32][1] );
     S[4][32][3] = gma2*S[1][32][3];
     S[2][32][4] = gma0*S[1][32][4]+     fak2* S[1][15][4];
     S[3][32][4] = gma1*S[1][32][4]+     fak2* S[1][14][4];
     S[4][32][4] = gma2*S[1][32][4]+     fak * S[1][32][1];
     S[2][33][2] = gma0*S[1][33][2]+     fak * (2.e0*S[1][20][2] + S[1][33][1] );
     S[3][33][2] = gma1*S[1][33][2]+     fak * S[1][16][2];
     S[4][33][2] = gma2*S[1][33][2]+     fak * S[1][14][2];
     S[2][33][3] = gma0*S[1][33][3]+     fak2* S[1][20][3];
     S[3][33][3] = gma1*S[1][33][3]+     fak * (S[1][16][3] + S[1][33][1] );
     S[4][33][3] = gma2*S[1][33][3]+     fak * S[1][14][3];
     S[2][33][4] = gma0*S[1][33][4]+     fak2* S[1][20][4];
     S[3][33][4] = gma1*S[1][33][4]+     fak * S[1][16][4];
     S[4][33][4] = gma2*S[1][33][4]+     fak * (S[1][14][4] + S[1][33][1] );
     S[2][34][2] = gma0*S[1][34][2]+     fak * (S[1][18][2] + S[1][34][1] );
     S[3][34][2] = gma1*S[1][34][2]+     fak2* S[1][20][2];
     S[4][34][2] = gma2*S[1][34][2]+     fak * S[1][15][2];
     S[2][34][3] = gma0*S[1][34][3]+     fak * S[1][18][3];
     S[3][34][3] = gma1*S[1][34][3]+     fak * (2.e0*S[1][20][3] + S[1][34][1] );
     S[4][34][3] = gma2*S[1][34][3]+     fak * S[1][15][3];
     S[2][34][4] = gma0*S[1][34][4]+     fak * S[1][18][4];
     S[3][34][4] = gma1*S[1][34][4]+     fak2* S[1][20][4];
     S[4][34][4] = gma2*S[1][34][4]+     fak * (S[1][15][4] + S[1][34][1] );
     S[2][35][2] = gma0*S[1][35][2]+     fak * (S[1][19][2] + S[1][35][1] );
     S[3][35][2] = gma1*S[1][35][2]+     fak * S[1][17][2];
     S[4][35][2] = gma2*S[1][35][2]+     fak2* S[1][20][2];
     S[2][35][3] = gma0*S[1][35][3]+     fak * S[1][19][3];
     S[3][35][3] = gma1*S[1][35][3]+     fak * (S[1][17][3] + S[1][35][1] );
     S[4][35][3] = gma2*S[1][35][3]+     fak2* S[1][20][3];
     S[2][35][4] = gma0*S[1][35][4]+     fak * S[1][19][4];
     S[3][35][4] = gma1*S[1][35][4]+     fak * S[1][17][4];
     S[4][35][4] = gma2*S[1][35][4]+     fak * (2.e0*S[1][20][4] + S[1][35][1] );
  }
  if ( _lmax_alpha>=2  ) {
     S[8][21][1] = gma0*S[2][21][1]+     fak * (S[1][21][1] + 4.e0*S[2][11][1] ) ;
     S[5][21][1] = gma1*S[2][21][1];
     S[6][21][1] = gma2*S[2][21][1];
     S[8][22][1] = gma0*S[2][22][1]+     fak * S[1][22][1];
     S[5][22][1] = gma1*S[2][22][1]+     fak4* S[2][12][1];
     S[6][22][1] = gma2*S[2][22][1];
     S[8][23][1] = gma0*S[2][23][1]+     fak * S[1][23][1];
     S[5][23][1] = gma1*S[2][23][1];
     S[6][23][1] = gma2*S[2][23][1]+     fak4* S[2][13][1];
     S[8][24][1] = gma0*S[2][24][1]+     fak * (S[1][24][1] + 3.e0*S[2][14][1] ) ;
     S[5][24][1] = gma1*S[2][24][1]+     fak * S[2][11][1];
     S[6][24][1] = gma2*S[2][24][1];
     S[8][25][1] = gma0*S[2][25][1]+     fak * (S[1][25][1] + S[2][12][1] ) ;
     S[5][25][1] = gma1*S[2][25][1]+     fak3* S[2][15][1];
     S[6][25][1] = gma2*S[2][25][1];
     S[8][26][1] = gma0*S[2][26][1]+     fak * (S[1][26][1] + 3.e0*S[2][16][1] ) ;
     S[5][26][1] = gma1*S[2][26][1];
     S[6][26][1] = gma2*S[2][26][1]+     fak * S[2][11][1];
     S[8][27][1] = gma0*S[2][27][1]+     fak * (S[1][27][1] + S[2][13][1] ) ;
     S[5][27][1] = gma1*S[2][27][1];
     S[6][27][1] = gma2*S[2][27][1]+     fak3* S[2][17][1];
     S[8][28][1] = gma0*S[2][28][1]+     fak * S[1][28][1];
     S[5][28][1] = gma1*S[2][28][1]+     fak3* S[2][18][1];
     S[6][28][1] = gma2*S[2][28][1]+     fak * S[2][12][1];
     S[8][29][1] = gma0*S[2][29][1]+     fak * S[1][29][1];
     S[5][29][1] = gma1*S[2][29][1]+     fak * S[2][13][1];
     S[6][29][1] = gma2*S[2][29][1]+     fak3* S[2][19][1];
     S[8][30][1] = gma0*S[2][30][1]+     fak * (S[1][30][1] + 2.e0*S[2][17][1] ) ;
     S[5][30][1] = gma1*S[2][30][1];
     S[6][30][1] = gma2*S[2][30][1]+     fak2* S[2][16][1];
     S[8][31][1] = gma0*S[2][31][1]+     fak * S[1][31][1];
     S[5][31][1] = gma1*S[2][31][1]+     fak2* S[2][19][1];
     S[6][31][1] = gma2*S[2][31][1]+     fak2* S[2][18][1];
     S[8][32][1] = gma0*S[2][32][1]+     fak * (S[1][32][1] + 2.e0*S[2][15][1] ) ;
     S[5][32][1] = gma1*S[2][32][1]+     fak2* S[2][14][1];
     S[6][32][1] = gma2*S[2][32][1];
     S[8][33][1] = gma0*S[2][33][1]+     fak * (S[1][33][1] + 2.e0*S[2][20][1] ) ;
     S[5][33][1] = gma1*S[2][33][1]+     fak * S[2][16][1];
     S[6][33][1] = gma2*S[2][33][1]+     fak * S[2][14][1];
     S[8][34][1] = gma0*S[2][34][1]+     fak * (S[1][34][1] + S[2][18][1] ) ;
     S[5][34][1] = gma1*S[2][34][1]+     fak2* S[2][20][1];
     S[6][34][1] = gma2*S[2][34][1]+     fak * S[2][15][1];
     S[8][35][1] = gma0*S[2][35][1]+     fak * (S[1][35][1] + S[2][19][1] ) ;
     S[5][35][1] = gma1*S[2][35][1]+     fak * S[2][17][1];
     S[6][35][1] = gma2*S[2][35][1]+     fak2* S[2][20][1];
     S[9][21][1] = gma1*S[3][21][1]+     fak * S[1][21][1];
     S[7][21][1] = gma2*S[3][21][1];
     S[9][22][1] = gma1*S[3][22][1]+     fak * (S[1][22][1] + 4.e0*S[3][12][1] ) ;
     S[7][22][1] = gma2*S[3][22][1];
     S[9][23][1] = gma1*S[3][23][1]+     fak * S[1][23][1];
     S[7][23][1] = gma2*S[3][23][1]+     fak4* S[3][13][1];
     S[9][24][1] = gma1*S[3][24][1]+     fak * (S[1][24][1] + S[3][11][1] ) ;
     S[7][24][1] = gma2*S[3][24][1];
     S[9][25][1] = gma1*S[3][25][1]+     fak * (S[1][25][1] + 3.e0*S[3][15][1] ) ;
     S[7][25][1] = gma2*S[3][25][1];
     S[9][26][1] = gma1*S[3][26][1]+     fak * S[1][26][1];
     S[7][26][1] = gma2*S[3][26][1]+     fak * S[3][11][1];
     S[9][27][1] = gma1*S[3][27][1]+     fak * S[1][27][1];
     S[7][27][1] = gma2*S[3][27][1]+     fak3* S[3][17][1];
     S[9][28][1] = gma1*S[3][28][1]+     fak * (S[1][28][1] + 3.e0*S[3][18][1] ) ;
     S[7][28][1] = gma2*S[3][28][1]+     fak * S[3][12][1];
     S[9][29][1] = gma1*S[3][29][1]+     fak * (S[1][29][1] + S[3][13][1] ) ;
     S[7][29][1] = gma2*S[3][29][1]+     fak3* S[3][19][1];
     S[9][30][1] = gma1*S[3][30][1]+     fak * S[1][30][1];
     S[7][30][1] = gma2*S[3][30][1]+     fak2* S[3][16][1];
     S[9][31][1] = gma1*S[3][31][1]+     fak * (S[1][31][1] + 2.e0*S[3][19][1] ) ;
     S[7][31][1] = gma2*S[3][31][1]+     fak2* S[3][18][1];
     S[9][32][1] = gma1*S[3][32][1]+     fak * (S[1][32][1] + 2.e0*S[3][14][1] ) ;
     S[7][32][1] = gma2*S[3][32][1];
     S[9][33][1] = gma1*S[3][33][1]+     fak * (S[1][33][1] + S[3][16][1] ) ;
     S[7][33][1] = gma2*S[3][33][1]+     fak * S[3][14][1];
     S[9][34][1] = gma1*S[3][34][1]+     fak * (S[1][34][1] + 2.e0*S[3][20][1] ) ;
     S[7][34][1] = gma2*S[3][34][1]+     fak * S[3][15][1];
     S[9][35][1] = gma1*S[3][35][1]+     fak * (S[1][35][1] + S[3][17][1] ) ;
     S[7][35][1] = gma2*S[3][35][1]+     fak2* S[3][20][1];
     S[10][21][1] = gma2*S[4][21][1]+     fak * S[1][21][1];
     S[10][22][1] = gma2*S[4][22][1]+     fak * S[1][22][1];
     S[10][23][1] = gma2*S[4][23][1]+     fak * (S[1][23][1] + 4.e0*S[4][13][1] ) ;
     S[10][24][1] = gma2*S[4][24][1]+     fak * S[1][24][1];
     S[10][25][1] = gma2*S[4][25][1]+     fak * S[1][25][1];
     S[10][26][1] = gma2*S[4][26][1]+     fak * (S[1][26][1] + S[4][11][1] ) ;
     S[10][27][1] = gma2*S[4][27][1]+     fak * (S[1][27][1] + 3.e0*S[4][17][1] ) ;
     S[10][28][1] = gma2*S[4][28][1]+     fak * (S[1][28][1] + S[4][12][1] ) ;
     S[10][29][1] = gma2*S[4][29][1]+     fak * (S[1][29][1] + 3.e0*S[4][19][1] ) ;
     S[10][30][1] = gma2*S[4][30][1]+     fak * (S[1][30][1] + 2.e0*S[4][16][1] ) ;
     S[10][31][1] = gma2*S[4][31][1]+     fak * (S[1][31][1] + 2.e0*S[4][18][1] ) ;
     S[10][32][1] = gma2*S[4][32][1]+     fak * S[1][32][1];
     S[10][33][1] = gma2*S[4][33][1]+     fak * (S[1][33][1] + S[4][14][1] ) ;
     S[10][34][1] = gma2*S[4][34][1]+     fak * (S[1][34][1] + S[4][15][1] ) ;
     S[10][35][1] = gma2*S[4][35][1]+     fak * (S[1][35][1] + 2.e0*S[4][20][1] ) ;
  }
  if ( _lmax_alpha>=1  &&  _lmax_gamma>=2 ) {
     S[2][21][5] = gma0*S[1][21][5]+     fak * (4.e0*S[1][11][5] + S[1][21][3] );
     S[3][21][5] = gma1*S[1][21][5]+     fak * S[1][21][2];
     S[4][21][5] = gma2*S[1][21][5];
     S[2][21][6] = gma0*S[1][21][6]+     fak * (4.e0*S[1][11][6] + S[1][21][4] );
     S[3][21][6] = gma1*S[1][21][6];
     S[4][21][6] = gma2*S[1][21][6]+     fak * S[1][21][2];
     S[2][21][7] = gma0*S[1][21][7]+     fak4* S[1][11][7];
     S[3][21][7] = gma1*S[1][21][7]+     fak * S[1][21][4];
     S[4][21][7] = gma2*S[1][21][7]+     fak * S[1][21][3];
     S[2][21][8] = gma0*S[1][21][8]+     fak2* (2.e0*S[1][11][8] + S[1][21][2] );
     S[3][21][8] = gma1*S[1][21][8];
     S[4][21][8] = gma2*S[1][21][8];
     S[2][21][9] = gma0*S[1][21][9]+     fak4* S[1][11][9];
     S[3][21][9] = gma1*S[1][21][9]+     fak2* S[1][21][3];
     S[4][21][9] = gma2*S[1][21][9];
     S[2][21][10] = gma0*S[1][21][10]+     fak4* S[1][11][10];
     S[3][21][10] = gma1*S[1][21][10];
     S[4][21][10] = gma2*S[1][21][10]+     fak2* S[1][21][4];
     S[2][22][5] = gma0*S[1][22][5]+     fak * S[1][22][3];
     S[3][22][5] = gma1*S[1][22][5]+     fak * (4.e0*S[1][12][5] + S[1][22][2] );
     S[4][22][5] = gma2*S[1][22][5];
     S[2][22][6] = gma0*S[1][22][6]+     fak * S[1][22][4];
     S[3][22][6] = gma1*S[1][22][6]+     fak4* S[1][12][6];
     S[4][22][6] = gma2*S[1][22][6]+     fak * S[1][22][2];
     S[2][22][7] = gma0*S[1][22][7];
     S[3][22][7] = gma1*S[1][22][7]+     fak * (4.e0*S[1][12][7] + S[1][22][4] );
     S[4][22][7] = gma2*S[1][22][7]+     fak * S[1][22][3];
     S[2][22][8] = gma0*S[1][22][8]+     fak2* S[1][22][2];
     S[3][22][8] = gma1*S[1][22][8]+     fak4* S[1][12][8];
     S[4][22][8] = gma2*S[1][22][8];
     S[2][22][9] = gma0*S[1][22][9];
     S[3][22][9] = gma1*S[1][22][9]+     fak2* (2.e0*S[1][12][9] + S[1][22][3] );
     S[4][22][9] = gma2*S[1][22][9];
     S[2][22][10] = gma0*S[1][22][10];
     S[3][22][10] = gma1*S[1][22][10]+     fak4* S[1][12][10];
     S[4][22][10] = gma2*S[1][22][10]+     fak2* S[1][22][4];
     S[2][23][5] = gma0*S[1][23][5]+     fak * S[1][23][3];
     S[3][23][5] = gma1*S[1][23][5]+     fak * S[1][23][2];
     S[4][23][5] = gma2*S[1][23][5]+     fak4* S[1][13][5];
     S[2][23][6] = gma0*S[1][23][6]+     fak * S[1][23][4];
     S[3][23][6] = gma1*S[1][23][6];
     S[4][23][6] = gma2*S[1][23][6]+     fak * (4.e0*S[1][13][6] + S[1][23][2] );
     S[2][23][7] = gma0*S[1][23][7];
     S[3][23][7] = gma1*S[1][23][7]+     fak * S[1][23][4];
     S[4][23][7] = gma2*S[1][23][7]+     fak * (4.e0*S[1][13][7] + S[1][23][3] );
     S[2][23][8] = gma0*S[1][23][8]+     fak2* S[1][23][2];
     S[3][23][8] = gma1*S[1][23][8];
     S[4][23][8] = gma2*S[1][23][8]+     fak4* S[1][13][8];
     S[2][23][9] = gma0*S[1][23][9];
     S[3][23][9] = gma1*S[1][23][9]+     fak2* S[1][23][3];
     S[4][23][9] = gma2*S[1][23][9]+     fak4* S[1][13][9];
     S[2][23][10] = gma0*S[1][23][10];
     S[3][23][10] = gma1*S[1][23][10];
     S[4][23][10] = gma2*S[1][23][10]+     fak2* (2.e0*S[1][13][10] + S[1][23][4] );
     S[2][24][5] = gma0*S[1][24][5]+     fak * (3.e0*S[1][14][5] + S[1][24][3] );
     S[3][24][5] = gma1*S[1][24][5]+     fak * (S[1][11][5] + S[1][24][2] );
     S[4][24][5] = gma2*S[1][24][5];
     S[2][24][6] = gma0*S[1][24][6]+     fak * (3.e0*S[1][14][6] + S[1][24][4] );
     S[3][24][6] = gma1*S[1][24][6]+     fak * S[1][11][6];
     S[4][24][6] = gma2*S[1][24][6]+     fak * S[1][24][2];
     S[2][24][7] = gma0*S[1][24][7]+     fak3* S[1][14][7];
     S[3][24][7] = gma1*S[1][24][7]+     fak * (S[1][11][7] + S[1][24][4] );
     S[4][24][7] = gma2*S[1][24][7]+     fak * S[1][24][3];
     S[2][24][8] = gma0*S[1][24][8]+     fak * (3.e0*S[1][14][8] + 2.e0*S[1][24][2] );
     S[3][24][8] = gma1*S[1][24][8]+     fak * S[1][11][8];
     S[4][24][8] = gma2*S[1][24][8];
     S[2][24][9] = gma0*S[1][24][9]+     fak3* S[1][14][9];
     S[3][24][9] = gma1*S[1][24][9]+     fak * (S[1][11][9] + 2.e0*S[1][24][3] );
     S[4][24][9] = gma2*S[1][24][9];
     S[2][24][10] = gma0*S[1][24][10]+     fak3* S[1][14][10];
     S[3][24][10] = gma1*S[1][24][10]+     fak * S[1][11][10];
     S[4][24][10] = gma2*S[1][24][10]+     fak2* S[1][24][4];
     S[2][25][5] = gma0*S[1][25][5]+     fak * (S[1][12][5] + S[1][25][3] );
     S[3][25][5] = gma1*S[1][25][5]+     fak * (3.e0*S[1][15][5] + S[1][25][2] );
     S[4][25][5] = gma2*S[1][25][5];
     S[2][25][6] = gma0*S[1][25][6]+     fak * (S[1][12][6] + S[1][25][4] );
     S[3][25][6] = gma1*S[1][25][6]+     fak3* S[1][15][6];
     S[4][25][6] = gma2*S[1][25][6]+     fak * S[1][25][2];
     S[2][25][7] = gma0*S[1][25][7]+     fak * S[1][12][7];
     S[3][25][7] = gma1*S[1][25][7]+     fak * (3.e0*S[1][15][7] + S[1][25][4] );
     S[4][25][7] = gma2*S[1][25][7]+     fak * S[1][25][3];
     S[2][25][8] = gma0*S[1][25][8]+     fak * (S[1][12][8] + 2.e0*S[1][25][2] );
     S[3][25][8] = gma1*S[1][25][8]+     fak3* S[1][15][8];
     S[4][25][8] = gma2*S[1][25][8];
     S[2][25][9] = gma0*S[1][25][9]+     fak * S[1][12][9];
     S[3][25][9] = gma1*S[1][25][9]+     fak * (3.e0*S[1][15][9] + 2.e0*S[1][25][3] );
     S[4][25][9] = gma2*S[1][25][9];
     S[2][25][10] = gma0*S[1][25][10]+     fak * S[1][12][10];
     S[3][25][10] = gma1*S[1][25][10]+     fak3* S[1][15][10];
     S[4][25][10] = gma2*S[1][25][10]+     fak2* S[1][25][4];
     S[2][26][5] = gma0*S[1][26][5]+     fak * (3.e0*S[1][16][5] + S[1][26][3] );
     S[3][26][5] = gma1*S[1][26][5]+     fak * S[1][26][2];
     S[4][26][5] = gma2*S[1][26][5]+     fak * S[1][11][5];
     S[2][26][6] = gma0*S[1][26][6]+     fak * (3.e0*S[1][16][6] + S[1][26][4] );
     S[3][26][6] = gma1*S[1][26][6];
     S[4][26][6] = gma2*S[1][26][6]+     fak * (S[1][11][6] + S[1][26][2] );
     S[2][26][7] = gma0*S[1][26][7]+     fak3* S[1][16][7];
     S[3][26][7] = gma1*S[1][26][7]+     fak * S[1][26][4];
     S[4][26][7] = gma2*S[1][26][7]+     fak * (S[1][11][7] + S[1][26][3] );
     S[2][26][8] = gma0*S[1][26][8]+     fak * (3.e0*S[1][16][8] + 2.e0*S[1][26][2] );
     S[3][26][8] = gma1*S[1][26][8];
     S[4][26][8] = gma2*S[1][26][8]+     fak * S[1][11][8];
     S[2][26][9] = gma0*S[1][26][9]+     fak3* S[1][16][9];
     S[3][26][9] = gma1*S[1][26][9]+     fak2* S[1][26][3];
     S[4][26][9] = gma2*S[1][26][9]+     fak * S[1][11][9];
     S[2][26][10] = gma0*S[1][26][10]+     fak3* S[1][16][10];
     S[3][26][10] = gma1*S[1][26][10];
     S[4][26][10] = gma2*S[1][26][10]+     fak * (S[1][11][10] + 2.e0*S[1][26][4] );
     S[2][27][5] = gma0*S[1][27][5]+     fak * (S[1][13][5] + S[1][27][3] );
     S[3][27][5] = gma1*S[1][27][5]+     fak * S[1][27][2];
     S[4][27][5] = gma2*S[1][27][5]+     fak3* S[1][17][5];
     S[2][27][6] = gma0*S[1][27][6]+     fak * (S[1][13][6] + S[1][27][4] );
     S[3][27][6] = gma1*S[1][27][6];
     S[4][27][6] = gma2*S[1][27][6]+     fak * (3.e0*S[1][17][6] + S[1][27][2] );
     S[2][27][7] = gma0*S[1][27][7]+     fak * S[1][13][7];
     S[3][27][7] = gma1*S[1][27][7]+     fak * S[1][27][4];
     S[4][27][7] = gma2*S[1][27][7]+     fak * (3.e0*S[1][17][7] + S[1][27][3] );
     S[2][27][8] = gma0*S[1][27][8]+     fak * (S[1][13][8] + 2.e0*S[1][27][2] );
     S[3][27][8] = gma1*S[1][27][8];
     S[4][27][8] = gma2*S[1][27][8]+     fak3* S[1][17][8];
     S[2][27][9] = gma0*S[1][27][9]+     fak * S[1][13][9];
     S[3][27][9] = gma1*S[1][27][9]+     fak2* S[1][27][3];
     S[4][27][9] = gma2*S[1][27][9]+     fak3* S[1][17][9];
     S[2][27][10] = gma0*S[1][27][10]+     fak * S[1][13][10];
     S[3][27][10] = gma1*S[1][27][10];
     S[4][27][10] = gma2*S[1][27][10]+     fak * (3.e0*S[1][17][10] + 2.e0*S[1][27][4] );
     S[2][28][5] = gma0*S[1][28][5]+     fak * S[1][28][3];
     S[3][28][5] = gma1*S[1][28][5]+     fak * (3.e0*S[1][18][5] + S[1][28][2] );
     S[4][28][5] = gma2*S[1][28][5]+     fak * S[1][12][5];
     S[2][28][6] = gma0*S[1][28][6]+     fak * S[1][28][4];
     S[3][28][6] = gma1*S[1][28][6]+     fak3* S[1][18][6];
     S[4][28][6] = gma2*S[1][28][6]+     fak * (S[1][12][6] + S[1][28][2] );
     S[2][28][7] = gma0*S[1][28][7];
     S[3][28][7] = gma1*S[1][28][7]+     fak * (3.e0*S[1][18][7] + S[1][28][4] );
     S[4][28][7] = gma2*S[1][28][7]+     fak * (S[1][12][7] + S[1][28][3] );
     S[2][28][8] = gma0*S[1][28][8]+     fak2* S[1][28][2];
     S[3][28][8] = gma1*S[1][28][8]+     fak3* S[1][18][8];
     S[4][28][8] = gma2*S[1][28][8]+     fak * S[1][12][8];
     S[2][28][9] = gma0*S[1][28][9];
     S[3][28][9] = gma1*S[1][28][9]+     fak * (3.e0*S[1][18][9] + 2.e0*S[1][28][3] );
     S[4][28][9] = gma2*S[1][28][9]+     fak * S[1][12][9];
     S[2][28][10] = gma0*S[1][28][10];
     S[3][28][10] = gma1*S[1][28][10]+     fak3* S[1][18][10];
     S[4][28][10] = gma2*S[1][28][10]+     fak * (S[1][12][10] + 2.e0*S[1][28][4] );
     S[2][29][5] = gma0*S[1][29][5]+     fak * S[1][29][3];
     S[3][29][5] = gma1*S[1][29][5]+     fak * (S[1][13][5] + S[1][29][2] );
     S[4][29][5] = gma2*S[1][29][5]+     fak3* S[1][19][5];
     S[2][29][6] = gma0*S[1][29][6]+     fak * S[1][29][4];
     S[3][29][6] = gma1*S[1][29][6]+     fak * S[1][13][6];
     S[4][29][6] = gma2*S[1][29][6]+     fak * (3.e0*S[1][19][6] + S[1][29][2] );
     S[2][29][7] = gma0*S[1][29][7];
     S[3][29][7] = gma1*S[1][29][7]+     fak * (S[1][13][7] + S[1][29][4] );
     S[4][29][7] = gma2*S[1][29][7]+     fak * (3.e0*S[1][19][7] + S[1][29][3] );
     S[2][29][8] = gma0*S[1][29][8]+     fak2* S[1][29][2];
     S[3][29][8] = gma1*S[1][29][8]+     fak * S[1][13][8];
     S[4][29][8] = gma2*S[1][29][8]+     fak3* S[1][19][8];
     S[2][29][9] = gma0*S[1][29][9];
     S[3][29][9] = gma1*S[1][29][9]+     fak * (S[1][13][9] + 2.e0*S[1][29][3] );
     S[4][29][9] = gma2*S[1][29][9]+     fak3* S[1][19][9];
     S[2][29][10] = gma0*S[1][29][10];
     S[3][29][10] = gma1*S[1][29][10]+     fak * S[1][13][10];
     S[4][29][10] = gma2*S[1][29][10]+     fak * (3.e0*S[1][19][10] + 2.e0*S[1][29][4] );
     S[2][30][5] = gma0*S[1][30][5]+     fak * (2.e0*S[1][17][5] + S[1][30][3] );
     S[3][30][5] = gma1*S[1][30][5]+     fak * S[1][30][2];
     S[4][30][5] = gma2*S[1][30][5]+     fak2* S[1][16][5];
     S[2][30][6] = gma0*S[1][30][6]+     fak * (2.e0*S[1][17][6] + S[1][30][4] );
     S[3][30][6] = gma1*S[1][30][6];
     S[4][30][6] = gma2*S[1][30][6]+     fak * (2.e0*S[1][16][6] + S[1][30][2] );
     S[2][30][7] = gma0*S[1][30][7]+     fak2* S[1][17][7];
     S[3][30][7] = gma1*S[1][30][7]+     fak * S[1][30][4];
     S[4][30][7] = gma2*S[1][30][7]+     fak * (2.e0*S[1][16][7] + S[1][30][3] );
     S[2][30][8] = gma0*S[1][30][8]+     fak2* (S[1][17][8] + S[1][30][2] );
     S[3][30][8] = gma1*S[1][30][8];
     S[4][30][8] = gma2*S[1][30][8]+     fak2* S[1][16][8];
     S[2][30][9] = gma0*S[1][30][9]+     fak2* S[1][17][9];
     S[3][30][9] = gma1*S[1][30][9]+     fak2* S[1][30][3];
     S[4][30][9] = gma2*S[1][30][9]+     fak2* S[1][16][9];
     S[2][30][10] = gma0*S[1][30][10]+     fak2* S[1][17][10];
     S[3][30][10] = gma1*S[1][30][10];
     S[4][30][10] = gma2*S[1][30][10]+     fak2* (S[1][16][10] + S[1][30][4] );
     S[2][31][5] = gma0*S[1][31][5]+     fak * S[1][31][3];
     S[3][31][5] = gma1*S[1][31][5]+     fak * (2.e0*S[1][19][5] + S[1][31][2] );
     S[4][31][5] = gma2*S[1][31][5]+     fak2* S[1][18][5];
     S[2][31][6] = gma0*S[1][31][6]+     fak * S[1][31][4];
     S[3][31][6] = gma1*S[1][31][6]+     fak2* S[1][19][6];
     S[4][31][6] = gma2*S[1][31][6]+     fak * (2.e0*S[1][18][6] + S[1][31][2] );
     S[2][31][7] = gma0*S[1][31][7];
     S[3][31][7] = gma1*S[1][31][7]+     fak * (2.e0*S[1][19][7] + S[1][31][4] );
     S[4][31][7] = gma2*S[1][31][7]+     fak * (2.e0*S[1][18][7] + S[1][31][3] );
     S[2][31][8] = gma0*S[1][31][8]+     fak2* S[1][31][2];
     S[3][31][8] = gma1*S[1][31][8]+     fak2* S[1][19][8];
     S[4][31][8] = gma2*S[1][31][8]+     fak2* S[1][18][8];
     S[2][31][9] = gma0*S[1][31][9];
     S[3][31][9] = gma1*S[1][31][9]+     fak2* (S[1][19][9] + S[1][31][3] );
     S[4][31][9] = gma2*S[1][31][9]+     fak2* S[1][18][9];
     S[2][31][10] = gma0*S[1][31][10];
     S[3][31][10] = gma1*S[1][31][10]+     fak2* S[1][19][10];
     S[4][31][10] = gma2*S[1][31][10]+     fak2* (S[1][18][10] + S[1][31][4] );
     S[2][32][5] = gma0*S[1][32][5]+     fak * (2.e0*S[1][15][5] + S[1][32][3] );
     S[3][32][5] = gma1*S[1][32][5]+     fak * (2.e0*S[1][14][5] + S[1][32][2] );
     S[4][32][5] = gma2*S[1][32][5];
     S[2][32][6] = gma0*S[1][32][6]+     fak * (2.e0*S[1][15][6] + S[1][32][4] );
     S[3][32][6] = gma1*S[1][32][6]+     fak2* S[1][14][6];
     S[4][32][6] = gma2*S[1][32][6]+     fak * S[1][32][2];
     S[2][32][7] = gma0*S[1][32][7]+     fak2* S[1][15][7];
     S[3][32][7] = gma1*S[1][32][7]+     fak * (2.e0*S[1][14][7] + S[1][32][4] );
     S[4][32][7] = gma2*S[1][32][7]+     fak * S[1][32][3];
     S[2][32][8] = gma0*S[1][32][8]+     fak2* (S[1][15][8] + S[1][32][2] );
     S[3][32][8] = gma1*S[1][32][8]+     fak2* S[1][14][8];
     S[4][32][8] = gma2*S[1][32][8];
     S[2][32][9] = gma0*S[1][32][9]+     fak2* S[1][15][9];
     S[3][32][9] = gma1*S[1][32][9]+     fak2* (S[1][14][9] + S[1][32][3] );
     S[4][32][9] = gma2*S[1][32][9];
     S[2][32][10] = gma0*S[1][32][10]+     fak2* S[1][15][10];
     S[3][32][10] = gma1*S[1][32][10]+     fak2* S[1][14][10];
     S[4][32][10] = gma2*S[1][32][10]+     fak2* S[1][32][4];
     S[2][33][5] = gma0*S[1][33][5]+     fak * (2.e0*S[1][20][5] + S[1][33][3] );
     S[3][33][5] = gma1*S[1][33][5]+     fak * (S[1][16][5] + S[1][33][2] );
     S[4][33][5] = gma2*S[1][33][5]+     fak * S[1][14][5];
     S[2][33][6] = gma0*S[1][33][6]+     fak * (2.e0*S[1][20][6] + S[1][33][4] );
     S[3][33][6] = gma1*S[1][33][6]+     fak * S[1][16][6];
     S[4][33][6] = gma2*S[1][33][6]+     fak * (S[1][14][6] + S[1][33][2] );
     S[2][33][7] = gma0*S[1][33][7]+     fak2* S[1][20][7];
     S[3][33][7] = gma1*S[1][33][7]+     fak * (S[1][16][7] + S[1][33][4] );
     S[4][33][7] = gma2*S[1][33][7]+     fak * (S[1][14][7] + S[1][33][3] );
     S[2][33][8] = gma0*S[1][33][8]+     fak2* (S[1][20][8] + S[1][33][2] );
     S[3][33][8] = gma1*S[1][33][8]+     fak * S[1][16][8];
     S[4][33][8] = gma2*S[1][33][8]+     fak * S[1][14][8];
     S[2][33][9] = gma0*S[1][33][9]+     fak2* S[1][20][9];
     S[3][33][9] = gma1*S[1][33][9]+     fak * (S[1][16][9] + 2.e0*S[1][33][3] );
     S[4][33][9] = gma2*S[1][33][9]+     fak * S[1][14][9];
     S[2][33][10] = gma0*S[1][33][10]+     fak2* S[1][20][10];
     S[3][33][10] = gma1*S[1][33][10]+     fak * S[1][16][10];
     S[4][33][10] = gma2*S[1][33][10]+     fak * (S[1][14][10] + 2.e0*S[1][33][4] );
     S[2][34][5] = gma0*S[1][34][5]+     fak * (S[1][18][5] + S[1][34][3] );
     S[3][34][5] = gma1*S[1][34][5]+     fak * (2.e0*S[1][20][5] + S[1][34][2] );
     S[4][34][5] = gma2*S[1][34][5]+     fak * S[1][15][5];
     S[2][34][6] = gma0*S[1][34][6]+     fak * (S[1][18][6] + S[1][34][4] );
     S[3][34][6] = gma1*S[1][34][6]+     fak2* S[1][20][6];
     S[4][34][6] = gma2*S[1][34][6]+     fak * (S[1][15][6] + S[1][34][2] );
     S[2][34][7] = gma0*S[1][34][7]+     fak * S[1][18][7];
     S[3][34][7] = gma1*S[1][34][7]+     fak * (2.e0*S[1][20][7] + S[1][34][4] );
     S[4][34][7] = gma2*S[1][34][7]+     fak * (S[1][15][7] + S[1][34][3] );
     S[2][34][8] = gma0*S[1][34][8]+     fak * (S[1][18][8] + 2.e0*S[1][34][2] );
     S[3][34][8] = gma1*S[1][34][8]+     fak2* S[1][20][8];
     S[4][34][8] = gma2*S[1][34][8]+     fak * S[1][15][8];
     S[2][34][9] = gma0*S[1][34][9]+     fak * S[1][18][9];
     S[3][34][9] = gma1*S[1][34][9]+     fak2* (S[1][20][9] + S[1][34][3] );
     S[4][34][9] = gma2*S[1][34][9]+     fak * S[1][15][9];
     S[2][34][10] = gma0*S[1][34][10]+     fak * S[1][18][10];
     S[3][34][10] = gma1*S[1][34][10]+     fak2* S[1][20][10];
     S[4][34][10] = gma2*S[1][34][10]+     fak * (S[1][15][10] + 2.e0*S[1][34][4] );
     S[2][35][5] = gma0*S[1][35][5]+     fak * (S[1][19][5] + S[1][35][3] );
     S[3][35][5] = gma1*S[1][35][5]+     fak * (S[1][17][5] + S[1][35][2] );
     S[4][35][5] = gma2*S[1][35][5]+     fak2* S[1][20][5];
     S[2][35][6] = gma0*S[1][35][6]+     fak * (S[1][19][6] + S[1][35][4] );
     S[3][35][6] = gma1*S[1][35][6]+     fak * S[1][17][6];
     S[4][35][6] = gma2*S[1][35][6]+     fak * (2.e0*S[1][20][6] + S[1][35][2] );
     S[2][35][7] = gma0*S[1][35][7]+     fak * S[1][19][7];
     S[3][35][7] = gma1*S[1][35][7]+     fak * (S[1][17][7] + S[1][35][4] );
     S[4][35][7] = gma2*S[1][35][7]+     fak * (2.e0*S[1][20][7] + S[1][35][3] );
     S[2][35][8] = gma0*S[1][35][8]+     fak * (S[1][19][8] + 2.e0*S[1][35][2] );
     S[3][35][8] = gma1*S[1][35][8]+     fak * S[1][17][8];
     S[4][35][8] = gma2*S[1][35][8]+     fak2* S[1][20][8];
     S[2][35][9] = gma0*S[1][35][9]+     fak * S[1][19][9];
     S[3][35][9] = gma1*S[1][35][9]+     fak * (S[1][17][9] + 2.e0*S[1][35][3] );
     S[4][35][9] = gma2*S[1][35][9]+     fak2* S[1][20][9];
     S[2][35][10] = gma0*S[1][35][10]+     fak * S[1][19][10];
     S[3][35][10] = gma1*S[1][35][10]+     fak * S[1][17][10];
     S[4][35][10] = gma2*S[1][35][10]+     fak2* (S[1][20][10] + S[1][35][4] );
  }
  if ( _lmax_alpha>=2  &&  _lmax_gamma>=1 ) {
     S[8][21][2] = gma0*S[2][21][2]+     fak * (S[1][21][2] + 4.e0*S[2][11][2] + S[2][21][1] );
     S[5][21][2] = gma1*S[2][21][2];
     S[6][21][2] = gma2*S[2][21][2];
     S[8][21][3] = gma0*S[2][21][3]+     fak * (S[1][21][3] + 4.e0*S[2][11][3] );
     S[5][21][3] = gma1*S[2][21][3]+     fak * S[2][21][1];
     S[6][21][3] = gma2*S[2][21][3];
     S[8][21][4] = gma0*S[2][21][4]+     fak * (S[1][21][4] + 4.e0*S[2][11][4] );
     S[5][21][4] = gma1*S[2][21][4];
     S[6][21][4] = gma2*S[2][21][4]+     fak * S[2][21][1];
     S[8][22][2] = gma0*S[2][22][2]+     fak * (S[1][22][2] + S[2][22][1] );
     S[5][22][2] = gma1*S[2][22][2]+     fak4* S[2][12][2];
     S[6][22][2] = gma2*S[2][22][2];
     S[8][22][3] = gma0*S[2][22][3]+     fak * S[1][22][3];
     S[5][22][3] = gma1*S[2][22][3]+     fak * (4.e0*S[2][12][3] + S[2][22][1] );
     S[6][22][3] = gma2*S[2][22][3];
     S[8][22][4] = gma0*S[2][22][4]+     fak * S[1][22][4];
     S[5][22][4] = gma1*S[2][22][4]+     fak4* S[2][12][4];
     S[6][22][4] = gma2*S[2][22][4]+     fak * S[2][22][1];
     S[8][23][2] = gma0*S[2][23][2]+     fak * (S[1][23][2] + S[2][23][1] );
     S[5][23][2] = gma1*S[2][23][2];
     S[6][23][2] = gma2*S[2][23][2]+     fak4* S[2][13][2];
     S[8][23][3] = gma0*S[2][23][3]+     fak * S[1][23][3];
     S[5][23][3] = gma1*S[2][23][3]+     fak * S[2][23][1];
     S[6][23][3] = gma2*S[2][23][3]+     fak4* S[2][13][3];
     S[8][23][4] = gma0*S[2][23][4]+     fak * S[1][23][4];
     S[5][23][4] = gma1*S[2][23][4];
     S[6][23][4] = gma2*S[2][23][4]+     fak * (4.e0*S[2][13][4] + S[2][23][1] );
     S[8][24][2] = gma0*S[2][24][2]+     fak * (S[1][24][2] + 3.e0*S[2][14][2] + S[2][24][1] );
     S[5][24][2] = gma1*S[2][24][2]+     fak * S[2][11][2];
     S[6][24][2] = gma2*S[2][24][2];
     S[8][24][3] = gma0*S[2][24][3]+     fak * (S[1][24][3] + 3.e0*S[2][14][3] );
     S[5][24][3] = gma1*S[2][24][3]+     fak * (S[2][11][3] + S[2][24][1] );
     S[6][24][3] = gma2*S[2][24][3];
     S[8][24][4] = gma0*S[2][24][4]+     fak * (S[1][24][4] + 3.e0*S[2][14][4] );
     S[5][24][4] = gma1*S[2][24][4]+     fak * S[2][11][4];
     S[6][24][4] = gma2*S[2][24][4]+     fak * S[2][24][1];
     S[8][25][2] = gma0*S[2][25][2]+     fak * (S[1][25][2] + S[2][12][2] + S[2][25][1] );
     S[5][25][2] = gma1*S[2][25][2]+     fak3* S[2][15][2];
     S[6][25][2] = gma2*S[2][25][2];
     S[8][25][3] = gma0*S[2][25][3]+     fak * (S[1][25][3] + S[2][12][3] );
     S[5][25][3] = gma1*S[2][25][3]+     fak * (3.e0*S[2][15][3] + S[2][25][1] );
     S[6][25][3] = gma2*S[2][25][3];
     S[8][25][4] = gma0*S[2][25][4]+     fak * (S[1][25][4] + S[2][12][4] );
     S[5][25][4] = gma1*S[2][25][4]+     fak3* S[2][15][4];
     S[6][25][4] = gma2*S[2][25][4]+     fak * S[2][25][1];
     S[8][26][2] = gma0*S[2][26][2]+     fak * (S[1][26][2] + 3.e0*S[2][16][2] + S[2][26][1] );
     S[5][26][2] = gma1*S[2][26][2];
     S[6][26][2] = gma2*S[2][26][2]+     fak * S[2][11][2];
     S[8][26][3] = gma0*S[2][26][3]+     fak * (S[1][26][3] + 3.e0*S[2][16][3] );
     S[5][26][3] = gma1*S[2][26][3]+     fak * S[2][26][1];
     S[6][26][3] = gma2*S[2][26][3]+     fak * S[2][11][3];
     S[8][26][4] = gma0*S[2][26][4]+     fak * (S[1][26][4] + 3.e0*S[2][16][4] );
     S[5][26][4] = gma1*S[2][26][4];
     S[6][26][4] = gma2*S[2][26][4]+     fak * (S[2][11][4] + S[2][26][1] );
     S[8][27][2] = gma0*S[2][27][2]+     fak * (S[1][27][2] + S[2][13][2] + S[2][27][1] );
     S[5][27][2] = gma1*S[2][27][2];
     S[6][27][2] = gma2*S[2][27][2]+     fak3* S[2][17][2];
     S[8][27][3] = gma0*S[2][27][3]+     fak * (S[1][27][3] + S[2][13][3] );
     S[5][27][3] = gma1*S[2][27][3]+     fak * S[2][27][1];
     S[6][27][3] = gma2*S[2][27][3]+     fak3* S[2][17][3];
     S[8][27][4] = gma0*S[2][27][4]+     fak * (S[1][27][4] + S[2][13][4] );
     S[5][27][4] = gma1*S[2][27][4];
     S[6][27][4] = gma2*S[2][27][4]+     fak * (3.e0*S[2][17][4] + S[2][27][1] );
     S[8][28][2] = gma0*S[2][28][2]+     fak * (S[1][28][2] + S[2][28][1] );
     S[5][28][2] = gma1*S[2][28][2]+     fak3* S[2][18][2];
     S[6][28][2] = gma2*S[2][28][2]+     fak * S[2][12][2];
     S[8][28][3] = gma0*S[2][28][3]+     fak * S[1][28][3];
     S[5][28][3] = gma1*S[2][28][3]+     fak * (3.e0*S[2][18][3] + S[2][28][1] );
     S[6][28][3] = gma2*S[2][28][3]+     fak * S[2][12][3];
     S[8][28][4] = gma0*S[2][28][4]+     fak * S[1][28][4];
     S[5][28][4] = gma1*S[2][28][4]+     fak3* S[2][18][4];
     S[6][28][4] = gma2*S[2][28][4]+     fak * (S[2][12][4] + S[2][28][1] );
     S[8][29][2] = gma0*S[2][29][2]+     fak * (S[1][29][2] + S[2][29][1] );
     S[5][29][2] = gma1*S[2][29][2]+     fak * S[2][13][2];
     S[6][29][2] = gma2*S[2][29][2]+     fak3* S[2][19][2];
     S[8][29][3] = gma0*S[2][29][3]+     fak * S[1][29][3];
     S[5][29][3] = gma1*S[2][29][3]+     fak * (S[2][13][3] + S[2][29][1] );
     S[6][29][3] = gma2*S[2][29][3]+     fak3* S[2][19][3];
     S[8][29][4] = gma0*S[2][29][4]+     fak * S[1][29][4];
     S[5][29][4] = gma1*S[2][29][4]+     fak * S[2][13][4];
     S[6][29][4] = gma2*S[2][29][4]+     fak * (3.e0*S[2][19][4] + S[2][29][1] );
     S[8][30][2] = gma0*S[2][30][2]+     fak * (S[1][30][2] + 2.e0*S[2][17][2] + S[2][30][1] );
     S[5][30][2] = gma1*S[2][30][2];
     S[6][30][2] = gma2*S[2][30][2]+     fak2* S[2][16][2];
     S[8][30][3] = gma0*S[2][30][3]+     fak * (S[1][30][3] + 2.e0*S[2][17][3] );
     S[5][30][3] = gma1*S[2][30][3]+     fak * S[2][30][1];
     S[6][30][3] = gma2*S[2][30][3]+     fak2* S[2][16][3];
     S[8][30][4] = gma0*S[2][30][4]+     fak * (S[1][30][4] + 2.e0*S[2][17][4] );
     S[5][30][4] = gma1*S[2][30][4];
     S[6][30][4] = gma2*S[2][30][4]+     fak * (2.e0*S[2][16][4] + S[2][30][1] );
     S[8][31][2] = gma0*S[2][31][2]+     fak * (S[1][31][2] + S[2][31][1] );
     S[5][31][2] = gma1*S[2][31][2]+     fak2* S[2][19][2];
     S[6][31][2] = gma2*S[2][31][2]+     fak2* S[2][18][2];
     S[8][31][3] = gma0*S[2][31][3]+     fak * S[1][31][3];
     S[5][31][3] = gma1*S[2][31][3]+     fak * (2.e0*S[2][19][3] + S[2][31][1] );
     S[6][31][3] = gma2*S[2][31][3]+     fak2* S[2][18][3];
     S[8][31][4] = gma0*S[2][31][4]+     fak * S[1][31][4];
     S[5][31][4] = gma1*S[2][31][4]+     fak2* S[2][19][4];
     S[6][31][4] = gma2*S[2][31][4]+     fak * (2.e0*S[2][18][4] + S[2][31][1] );
     S[8][32][2] = gma0*S[2][32][2]+     fak * (S[1][32][2] + 2.e0*S[2][15][2] + S[2][32][1] );
     S[5][32][2] = gma1*S[2][32][2]+     fak2* S[2][14][2];
     S[6][32][2] = gma2*S[2][32][2];
     S[8][32][3] = gma0*S[2][32][3]+     fak * (S[1][32][3] + 2.e0*S[2][15][3] );
     S[5][32][3] = gma1*S[2][32][3]+     fak * (2.e0*S[2][14][3] + S[2][32][1] );
     S[6][32][3] = gma2*S[2][32][3];
     S[8][32][4] = gma0*S[2][32][4]+     fak * (S[1][32][4] + 2.e0*S[2][15][4] );
     S[5][32][4] = gma1*S[2][32][4]+     fak2* S[2][14][4];
     S[6][32][4] = gma2*S[2][32][4]+     fak * S[2][32][1];
     S[8][33][2] = gma0*S[2][33][2]+     fak * (S[1][33][2] + 2.e0*S[2][20][2] + S[2][33][1] );
     S[5][33][2] = gma1*S[2][33][2]+     fak * S[2][16][2];
     S[6][33][2] = gma2*S[2][33][2]+     fak * S[2][14][2];
     S[8][33][3] = gma0*S[2][33][3]+     fak * (S[1][33][3] + 2.e0*S[2][20][3] );
     S[5][33][3] = gma1*S[2][33][3]+     fak * (S[2][16][3] + S[2][33][1] );
     S[6][33][3] = gma2*S[2][33][3]+     fak * S[2][14][3];
     S[8][33][4] = gma0*S[2][33][4]+     fak * (S[1][33][4] + 2.e0*S[2][20][4] );
     S[5][33][4] = gma1*S[2][33][4]+     fak * S[2][16][4];
     S[6][33][4] = gma2*S[2][33][4]+     fak * (S[2][14][4] + S[2][33][1] );
     S[8][34][2] = gma0*S[2][34][2]+     fak * (S[1][34][2] + S[2][18][2] + S[2][34][1] );
     S[5][34][2] = gma1*S[2][34][2]+     fak2* S[2][20][2];
     S[6][34][2] = gma2*S[2][34][2]+     fak * S[2][15][2];
     S[8][34][3] = gma0*S[2][34][3]+     fak * (S[1][34][3] + S[2][18][3] );
     S[5][34][3] = gma1*S[2][34][3]+     fak * (2.e0*S[2][20][3] + S[2][34][1] );
     S[6][34][3] = gma2*S[2][34][3]+     fak * S[2][15][3];
     S[8][34][4] = gma0*S[2][34][4]+     fak * (S[1][34][4] + S[2][18][4] );
     S[5][34][4] = gma1*S[2][34][4]+     fak2* S[2][20][4];
     S[6][34][4] = gma2*S[2][34][4]+     fak * (S[2][15][4] + S[2][34][1] );
     S[8][35][2] = gma0*S[2][35][2]+     fak * (S[1][35][2] + S[2][19][2] + S[2][35][1] );
     S[5][35][2] = gma1*S[2][35][2]+     fak * S[2][17][2];
     S[6][35][2] = gma2*S[2][35][2]+     fak2* S[2][20][2];
     S[8][35][3] = gma0*S[2][35][3]+     fak * (S[1][35][3] + S[2][19][3] );
     S[5][35][3] = gma1*S[2][35][3]+     fak * (S[2][17][3] + S[2][35][1] );
     S[6][35][3] = gma2*S[2][35][3]+     fak2* S[2][20][3];
     S[8][35][4] = gma0*S[2][35][4]+     fak * (S[1][35][4] + S[2][19][4] );
     S[5][35][4] = gma1*S[2][35][4]+     fak * S[2][17][4];
     S[6][35][4] = gma2*S[2][35][4]+     fak * (2.e0*S[2][20][4] + S[2][35][1] );
     S[9][21][2] = gma1*S[3][21][2]+     fak * S[1][21][2];
     S[7][21][2] = gma2*S[3][21][2];
     S[9][21][3] = gma1*S[3][21][3]+     fak * (S[1][21][3] + S[3][21][1] );
     S[7][21][3] = gma2*S[3][21][3];
     S[9][21][4] = gma1*S[3][21][4]+     fak * S[1][21][4];
     S[7][21][4] = gma2*S[3][21][4]+     fak * S[3][21][1];
     S[9][22][2] = gma1*S[3][22][2]+     fak * (S[1][22][2] + 4.e0*S[3][12][2] );
     S[7][22][2] = gma2*S[3][22][2];
     S[9][22][3] = gma1*S[3][22][3]+     fak * (S[1][22][3] + 4.e0*S[3][12][3] + S[3][22][1] );
     S[7][22][3] = gma2*S[3][22][3];
     S[9][22][4] = gma1*S[3][22][4]+     fak * (S[1][22][4] + 4.e0*S[3][12][4] );
     S[7][22][4] = gma2*S[3][22][4]+     fak * S[3][22][1];
     S[9][23][2] = gma1*S[3][23][2]+     fak * S[1][23][2];
     S[7][23][2] = gma2*S[3][23][2]+     fak4* S[3][13][2];
     S[9][23][3] = gma1*S[3][23][3]+     fak * (S[1][23][3] + S[3][23][1] );
     S[7][23][3] = gma2*S[3][23][3]+     fak4* S[3][13][3];
     S[9][23][4] = gma1*S[3][23][4]+     fak * S[1][23][4];
     S[7][23][4] = gma2*S[3][23][4]+     fak * (4.e0*S[3][13][4] + S[3][23][1] );
     S[9][24][2] = gma1*S[3][24][2]+     fak * (S[1][24][2] + S[3][11][2] );
     S[7][24][2] = gma2*S[3][24][2];
     S[9][24][3] = gma1*S[3][24][3]+     fak * (S[1][24][3] + S[3][11][3] + S[3][24][1] );
     S[7][24][3] = gma2*S[3][24][3];
     S[9][24][4] = gma1*S[3][24][4]+     fak * (S[1][24][4] + S[3][11][4] );
     S[7][24][4] = gma2*S[3][24][4]+     fak * S[3][24][1];
     S[9][25][2] = gma1*S[3][25][2]+     fak * (S[1][25][2] + 3.e0*S[3][15][2] );
     S[7][25][2] = gma2*S[3][25][2];
     S[9][25][3] = gma1*S[3][25][3]+     fak * (S[1][25][3] + 3.e0*S[3][15][3] + S[3][25][1] );
     S[7][25][3] = gma2*S[3][25][3];
     S[9][25][4] = gma1*S[3][25][4]+     fak * (S[1][25][4] + 3.e0*S[3][15][4] );
     S[7][25][4] = gma2*S[3][25][4]+     fak * S[3][25][1];
     S[9][26][2] = gma1*S[3][26][2]+     fak * S[1][26][2];
     S[7][26][2] = gma2*S[3][26][2]+     fak * S[3][11][2];
     S[9][26][3] = gma1*S[3][26][3]+     fak * (S[1][26][3] + S[3][26][1] );
     S[7][26][3] = gma2*S[3][26][3]+     fak * S[3][11][3];
     S[9][26][4] = gma1*S[3][26][4]+     fak * S[1][26][4];
     S[7][26][4] = gma2*S[3][26][4]+     fak * (S[3][11][4] + S[3][26][1] );
     S[9][27][2] = gma1*S[3][27][2]+     fak * S[1][27][2];
     S[7][27][2] = gma2*S[3][27][2]+     fak3* S[3][17][2];
     S[9][27][3] = gma1*S[3][27][3]+     fak * (S[1][27][3] + S[3][27][1] );
     S[7][27][3] = gma2*S[3][27][3]+     fak3* S[3][17][3];
     S[9][27][4] = gma1*S[3][27][4]+     fak * S[1][27][4];
     S[7][27][4] = gma2*S[3][27][4]+     fak * (3.e0*S[3][17][4] + S[3][27][1] );
     S[9][28][2] = gma1*S[3][28][2]+     fak * (S[1][28][2] + 3.e0*S[3][18][2] );
     S[7][28][2] = gma2*S[3][28][2]+     fak * S[3][12][2];
     S[9][28][3] = gma1*S[3][28][3]+     fak * (S[1][28][3] + 3.e0*S[3][18][3] + S[3][28][1] );
     S[7][28][3] = gma2*S[3][28][3]+     fak * S[3][12][3];
     S[9][28][4] = gma1*S[3][28][4]+     fak * (S[1][28][4] + 3.e0*S[3][18][4] );
     S[7][28][4] = gma2*S[3][28][4]+     fak * (S[3][12][4] + S[3][28][1] );
     S[9][29][2] = gma1*S[3][29][2]+     fak * (S[1][29][2] + S[3][13][2] );
     S[7][29][2] = gma2*S[3][29][2]+     fak3* S[3][19][2];
     S[9][29][3] = gma1*S[3][29][3]+     fak * (S[1][29][3] + S[3][13][3] + S[3][29][1] );
     S[7][29][3] = gma2*S[3][29][3]+     fak3* S[3][19][3];
     S[9][29][4] = gma1*S[3][29][4]+     fak * (S[1][29][4] + S[3][13][4] );
     S[7][29][4] = gma2*S[3][29][4]+     fak * (3.e0*S[3][19][4] + S[3][29][1] );
     S[9][30][2] = gma1*S[3][30][2]+     fak * S[1][30][2];
     S[7][30][2] = gma2*S[3][30][2]+     fak2* S[3][16][2];
     S[9][30][3] = gma1*S[3][30][3]+     fak * (S[1][30][3] + S[3][30][1] );
     S[7][30][3] = gma2*S[3][30][3]+     fak2* S[3][16][3];
     S[9][30][4] = gma1*S[3][30][4]+     fak * S[1][30][4];
     S[7][30][4] = gma2*S[3][30][4]+     fak * (2.e0*S[3][16][4] + S[3][30][1] );
     S[9][31][2] = gma1*S[3][31][2]+     fak * (S[1][31][2] + 2.e0*S[3][19][2] );
     S[7][31][2] = gma2*S[3][31][2]+     fak2* S[3][18][2];
     S[9][31][3] = gma1*S[3][31][3]+     fak * (S[1][31][3] + 2.e0*S[3][19][3] + S[3][31][1] );
     S[7][31][3] = gma2*S[3][31][3]+     fak2* S[3][18][3];
     S[9][31][4] = gma1*S[3][31][4]+     fak * (S[1][31][4] + 2.e0*S[3][19][4] );
     S[7][31][4] = gma2*S[3][31][4]+     fak * (2.e0*S[3][18][4] + S[3][31][1] );
     S[9][32][2] = gma1*S[3][32][2]+     fak * (S[1][32][2] + 2.e0*S[3][14][2] );
     S[7][32][2] = gma2*S[3][32][2];
     S[9][32][3] = gma1*S[3][32][3]+     fak * (S[1][32][3] + 2.e0*S[3][14][3] + S[3][32][1] );
     S[7][32][3] = gma2*S[3][32][3];
     S[9][32][4] = gma1*S[3][32][4]+     fak * (S[1][32][4] + 2.e0*S[3][14][4] );
     S[7][32][4] = gma2*S[3][32][4]+     fak * S[3][32][1];
     S[9][33][2] = gma1*S[3][33][2]+     fak * (S[1][33][2] + S[3][16][2] );
     S[7][33][2] = gma2*S[3][33][2]+     fak * S[3][14][2];
     S[9][33][3] = gma1*S[3][33][3]+     fak * (S[1][33][3] + S[3][16][3] + S[3][33][1] );
     S[7][33][3] = gma2*S[3][33][3]+     fak * S[3][14][3];
     S[9][33][4] = gma1*S[3][33][4]+     fak * (S[1][33][4] + S[3][16][4] );
     S[7][33][4] = gma2*S[3][33][4]+     fak * (S[3][14][4] + S[3][33][1] );
     S[9][34][2] = gma1*S[3][34][2]+     fak * (S[1][34][2] + 2.e0*S[3][20][2] );
     S[7][34][2] = gma2*S[3][34][2]+     fak * S[3][15][2];
     S[9][34][3] = gma1*S[3][34][3]+     fak * (S[1][34][3] + 2.e0*S[3][20][3] + S[3][34][1] );
     S[7][34][3] = gma2*S[3][34][3]+     fak * S[3][15][3];
     S[9][34][4] = gma1*S[3][34][4]+     fak * (S[1][34][4] + 2.e0*S[3][20][4] );
     S[7][34][4] = gma2*S[3][34][4]+     fak * (S[3][15][4] + S[3][34][1] );
     S[9][35][2] = gma1*S[3][35][2]+     fak * (S[1][35][2] + S[3][17][2] );
     S[7][35][2] = gma2*S[3][35][2]+     fak2* S[3][20][2];
     S[9][35][3] = gma1*S[3][35][3]+     fak * (S[1][35][3] + S[3][17][3] + S[3][35][1] );
     S[7][35][3] = gma2*S[3][35][3]+     fak2* S[3][20][3];
     S[9][35][4] = gma1*S[3][35][4]+     fak * (S[1][35][4] + S[3][17][4] );
     S[7][35][4] = gma2*S[3][35][4]+     fak * (2.e0*S[3][20][4] + S[3][35][1] );
     S[10][21][2] = gma2*S[4][21][2]+     fak * S[1][21][2];
     S[10][21][3] = gma2*S[4][21][3]+     fak * S[1][21][3];
     S[10][21][4] = gma2*S[4][21][4]+     fak * (S[1][21][4] + S[4][21][1] );
     S[10][22][2] = gma2*S[4][22][2]+     fak * S[1][22][2];
     S[10][22][3] = gma2*S[4][22][3]+     fak * S[1][22][3];
     S[10][22][4] = gma2*S[4][22][4]+     fak * (S[1][22][4] + S[4][22][1] );
     S[10][23][2] = gma2*S[4][23][2]+     fak * (S[1][23][2] + 4.e0*S[4][13][2] );
     S[10][23][3] = gma2*S[4][23][3]+     fak * (S[1][23][3] + 4.e0*S[4][13][3] );
     S[10][23][4] = gma2*S[4][23][4]+     fak * (S[1][23][4] + 4.e0*S[4][13][4] + S[4][23][1] );
     S[10][24][2] = gma2*S[4][24][2]+     fak * S[1][24][2];
     S[10][24][3] = gma2*S[4][24][3]+     fak * S[1][24][3];
     S[10][24][4] = gma2*S[4][24][4]+     fak * (S[1][24][4] + S[4][24][1] );
     S[10][25][2] = gma2*S[4][25][2]+     fak * S[1][25][2];
     S[10][25][3] = gma2*S[4][25][3]+     fak * S[1][25][3];
     S[10][25][4] = gma2*S[4][25][4]+     fak * (S[1][25][4] + S[4][25][1] );
     S[10][26][2] = gma2*S[4][26][2]+     fak * (S[1][26][2] + S[4][11][2] );
     S[10][26][3] = gma2*S[4][26][3]+     fak * (S[1][26][3] + S[4][11][3] );
     S[10][26][4] = gma2*S[4][26][4]+     fak * (S[1][26][4] + S[4][11][4] + S[4][26][1] );
     S[10][27][2] = gma2*S[4][27][2]+     fak * (S[1][27][2] + 3.e0*S[4][17][2] );
     S[10][27][3] = gma2*S[4][27][3]+     fak * (S[1][27][3] + 3.e0*S[4][17][3] );
     S[10][27][4] = gma2*S[4][27][4]+     fak * (S[1][27][4] + 3.e0*S[4][17][4] + S[4][27][1] );
     S[10][28][2] = gma2*S[4][28][2]+     fak * (S[1][28][2] + S[4][12][2] );
     S[10][28][3] = gma2*S[4][28][3]+     fak * (S[1][28][3] + S[4][12][3] );
     S[10][28][4] = gma2*S[4][28][4]+     fak * (S[1][28][4] + S[4][12][4] + S[4][28][1] );
     S[10][29][2] = gma2*S[4][29][2]+     fak * (S[1][29][2] + 3.e0*S[4][19][2] );
     S[10][29][3] = gma2*S[4][29][3]+     fak * (S[1][29][3] + 3.e0*S[4][19][3] );
     S[10][29][4] = gma2*S[4][29][4]+     fak * (S[1][29][4] + 3.e0*S[4][19][4] + S[4][29][1] );
     S[10][30][2] = gma2*S[4][30][2]+     fak * (S[1][30][2] + 2.e0*S[4][16][2] );
     S[10][30][3] = gma2*S[4][30][3]+     fak * (S[1][30][3] + 2.e0*S[4][16][3] );
     S[10][30][4] = gma2*S[4][30][4]+     fak * (S[1][30][4] + 2.e0*S[4][16][4] + S[4][30][1] );
     S[10][31][2] = gma2*S[4][31][2]+     fak * (S[1][31][2] + 2.e0*S[4][18][2] );
     S[10][31][3] = gma2*S[4][31][3]+     fak * (S[1][31][3] + 2.e0*S[4][18][3] );
     S[10][31][4] = gma2*S[4][31][4]+     fak * (S[1][31][4] + 2.e0*S[4][18][4] + S[4][31][1] );
     S[10][32][2] = gma2*S[4][32][2]+     fak * S[1][32][2];
     S[10][32][3] = gma2*S[4][32][3]+     fak * S[1][32][3];
     S[10][32][4] = gma2*S[4][32][4]+     fak * (S[1][32][4] + S[4][32][1] );
     S[10][33][2] = gma2*S[4][33][2]+     fak * (S[1][33][2] + S[4][14][2] );
     S[10][33][3] = gma2*S[4][33][3]+     fak * (S[1][33][3] + S[4][14][3] );
     S[10][33][4] = gma2*S[4][33][4]+     fak * (S[1][33][4] + S[4][14][4] + S[4][33][1] );
     S[10][34][2] = gma2*S[4][34][2]+     fak * (S[1][34][2] + S[4][15][2] );
     S[10][34][3] = gma2*S[4][34][3]+     fak * (S[1][34][3] + S[4][15][3] );
     S[10][34][4] = gma2*S[4][34][4]+     fak * (S[1][34][4] + S[4][15][4] + S[4][34][1] );
     S[10][35][2] = gma2*S[4][35][2]+     fak * (S[1][35][2] + 2.e0*S[4][20][2] );
     S[10][35][3] = gma2*S[4][35][3]+     fak * (S[1][35][3] + 2.e0*S[4][20][3] );
     S[10][35][4] = gma2*S[4][35][4]+     fak * (S[1][35][4] + 2.e0*S[4][20][4] + S[4][35][1] );
  }
  if ( _lmax_alpha>=2  &&  _lmax_gamma>=2 ) {
     S[8][21][5] = gma0*S[2][21][5]+     fak * (S[1][21][5] + 4.e0*S[2][11][5] + S[2][21][3] );
     S[5][21][5] = gma1*S[2][21][5]+     fak * S[2][21][2];
     S[6][21][5] = gma2*S[2][21][5];
     S[8][21][6] = gma0*S[2][21][6]+     fak * (S[1][21][6] + 4.e0*S[2][11][6] + S[2][21][4] );
     S[5][21][6] = gma1*S[2][21][6];
     S[6][21][6] = gma2*S[2][21][6]+     fak * S[2][21][2];
     S[8][21][7] = gma0*S[2][21][7]+     fak * (S[1][21][7] + 4.e0*S[2][11][7] );
     S[5][21][7] = gma1*S[2][21][7]+     fak * S[2][21][4];
     S[6][21][7] = gma2*S[2][21][7]+     fak * S[2][21][3];
     S[8][21][8] = gma0*S[2][21][8]+     fak * (S[1][21][8] + 4.e0*S[2][11][8] + 2.e0*S[2][21][2] );
     S[5][21][8] = gma1*S[2][21][8];
     S[6][21][8] = gma2*S[2][21][8];
     S[8][21][9] = gma0*S[2][21][9]+     fak * (S[1][21][9] + 4.e0*S[2][11][9] );
     S[5][21][9] = gma1*S[2][21][9]+     fak2* S[2][21][3];
     S[6][21][9] = gma2*S[2][21][9];
     S[8][21][10] = gma0*S[2][21][10]+     fak * (S[1][21][10] + 4.e0*S[2][11][10] );
     S[5][21][10] = gma1*S[2][21][10];
     S[6][21][10] = gma2*S[2][21][10]+     fak2* S[2][21][4];
     S[8][22][5] = gma0*S[2][22][5]+     fak * (S[1][22][5] + S[2][22][3] );
     S[5][22][5] = gma1*S[2][22][5]+     fak * (4.e0*S[2][12][5] + S[2][22][2] );
     S[6][22][5] = gma2*S[2][22][5];
     S[8][22][6] = gma0*S[2][22][6]+     fak * (S[1][22][6] + S[2][22][4] );
     S[5][22][6] = gma1*S[2][22][6]+     fak4* S[2][12][6];
     S[6][22][6] = gma2*S[2][22][6]+     fak * S[2][22][2];
     S[8][22][7] = gma0*S[2][22][7]+     fak * S[1][22][7];
     S[5][22][7] = gma1*S[2][22][7]+     fak * (4.e0*S[2][12][7] + S[2][22][4] );
     S[6][22][7] = gma2*S[2][22][7]+     fak * S[2][22][3];
     S[8][22][8] = gma0*S[2][22][8]+     fak * (S[1][22][8] + 2.e0*S[2][22][2] );
     S[5][22][8] = gma1*S[2][22][8]+     fak4* S[2][12][8];
     S[6][22][8] = gma2*S[2][22][8];
     S[8][22][9] = gma0*S[2][22][9]+     fak * S[1][22][9];
     S[5][22][9] = gma1*S[2][22][9]+     fak2* (2.e0*S[2][12][9] + S[2][22][3] );
     S[6][22][9] = gma2*S[2][22][9];
     S[8][22][10] = gma0*S[2][22][10]+     fak * S[1][22][10];
     S[5][22][10] = gma1*S[2][22][10]+     fak4* S[2][12][10];
     S[6][22][10] = gma2*S[2][22][10]+     fak2* S[2][22][4];
     S[8][23][5] = gma0*S[2][23][5]+     fak * (S[1][23][5] + S[2][23][3] );
     S[5][23][5] = gma1*S[2][23][5]+     fak * S[2][23][2];
     S[6][23][5] = gma2*S[2][23][5]+     fak4* S[2][13][5];
     S[8][23][6] = gma0*S[2][23][6]+     fak * (S[1][23][6] + S[2][23][4] );
     S[5][23][6] = gma1*S[2][23][6];
     S[6][23][6] = gma2*S[2][23][6]+     fak * (4.e0*S[2][13][6] + S[2][23][2] );
     S[8][23][7] = gma0*S[2][23][7]+     fak * S[1][23][7];
     S[5][23][7] = gma1*S[2][23][7]+     fak * S[2][23][4];
     S[6][23][7] = gma2*S[2][23][7]+     fak * (4.e0*S[2][13][7] + S[2][23][3] );
     S[8][23][8] = gma0*S[2][23][8]+     fak * (S[1][23][8] + 2.e0*S[2][23][2] );
     S[5][23][8] = gma1*S[2][23][8];
     S[6][23][8] = gma2*S[2][23][8]+     fak4* S[2][13][8];
     S[8][23][9] = gma0*S[2][23][9]+     fak * S[1][23][9];
     S[5][23][9] = gma1*S[2][23][9]+     fak2* S[2][23][3];
     S[6][23][9] = gma2*S[2][23][9]+     fak4* S[2][13][9];
     S[8][23][10] = gma0*S[2][23][10]+     fak * S[1][23][10];
     S[5][23][10] = gma1*S[2][23][10];
     S[6][23][10] = gma2*S[2][23][10]+     fak2* (2.e0*S[2][13][10] + S[2][23][4] );
     S[8][24][5] = gma0*S[2][24][5]+     fak * (S[1][24][5] + 3.e0*S[2][14][5] + S[2][24][3] );
     S[5][24][5] = gma1*S[2][24][5]+     fak * (S[2][11][5] + S[2][24][2] );
     S[6][24][5] = gma2*S[2][24][5];
     S[8][24][6] = gma0*S[2][24][6]+     fak * (S[1][24][6] + 3.e0*S[2][14][6] + S[2][24][4] );
     S[5][24][6] = gma1*S[2][24][6]+     fak * S[2][11][6];
     S[6][24][6] = gma2*S[2][24][6]+     fak * S[2][24][2];
     S[8][24][7] = gma0*S[2][24][7]+     fak * (S[1][24][7] + 3.e0*S[2][14][7] );
     S[5][24][7] = gma1*S[2][24][7]+     fak * (S[2][11][7] + S[2][24][4] );
     S[6][24][7] = gma2*S[2][24][7]+     fak * S[2][24][3];
     S[8][24][8] = gma0*S[2][24][8]+     fak * (S[1][24][8] + 3.e0*S[2][14][8] + 2.e0*S[2][24][2] );
     S[5][24][8] = gma1*S[2][24][8]+     fak * S[2][11][8];
     S[6][24][8] = gma2*S[2][24][8];
     S[8][24][9] = gma0*S[2][24][9]+     fak * (S[1][24][9] + 3.e0*S[2][14][9] );
     S[5][24][9] = gma1*S[2][24][9]+     fak * (S[2][11][9] + 2.e0*S[2][24][3] );
     S[6][24][9] = gma2*S[2][24][9];
     S[8][24][10] = gma0*S[2][24][10]+     fak * (S[1][24][10] + 3.e0*S[2][14][10] );
     S[5][24][10] = gma1*S[2][24][10]+     fak * S[2][11][10];
     S[6][24][10] = gma2*S[2][24][10]+     fak2* S[2][24][4];
     S[8][25][5] = gma0*S[2][25][5]+     fak * (S[1][25][5] + S[2][12][5] + S[2][25][3] );
     S[5][25][5] = gma1*S[2][25][5]+     fak * (3.e0*S[2][15][5] + S[2][25][2] );
     S[6][25][5] = gma2*S[2][25][5];
     S[8][25][6] = gma0*S[2][25][6]+     fak * (S[1][25][6] + S[2][12][6] + S[2][25][4] );
     S[5][25][6] = gma1*S[2][25][6]+     fak3* S[2][15][6];
     S[6][25][6] = gma2*S[2][25][6]+     fak * S[2][25][2];
     S[8][25][7] = gma0*S[2][25][7]+     fak * (S[1][25][7] + S[2][12][7] );
     S[5][25][7] = gma1*S[2][25][7]+     fak * (3.e0*S[2][15][7] + S[2][25][4] );
     S[6][25][7] = gma2*S[2][25][7]+     fak * S[2][25][3];
     S[8][25][8] = gma0*S[2][25][8]+     fak * (S[1][25][8] + S[2][12][8] + 2.e0*S[2][25][2] );
     S[5][25][8] = gma1*S[2][25][8]+     fak3* S[2][15][8];
     S[6][25][8] = gma2*S[2][25][8];
     S[8][25][9] = gma0*S[2][25][9]+     fak * (S[1][25][9] + S[2][12][9] );
     S[5][25][9] = gma1*S[2][25][9]+     fak * (3.e0*S[2][15][9] + 2.e0*S[2][25][3] );
     S[6][25][9] = gma2*S[2][25][9];
     S[8][25][10] = gma0*S[2][25][10]+     fak * (S[1][25][10] + S[2][12][10] );
     S[5][25][10] = gma1*S[2][25][10]+     fak3* S[2][15][10];
     S[6][25][10] = gma2*S[2][25][10]+     fak2* S[2][25][4];
     S[8][26][5] = gma0*S[2][26][5]+     fak * (S[1][26][5] + 3.e0*S[2][16][5] + S[2][26][3] );
     S[5][26][5] = gma1*S[2][26][5]+     fak * S[2][26][2];
     S[6][26][5] = gma2*S[2][26][5]+     fak * S[2][11][5];
     S[8][26][6] = gma0*S[2][26][6]+     fak * (S[1][26][6] + 3.e0*S[2][16][6] + S[2][26][4] );
     S[5][26][6] = gma1*S[2][26][6];
     S[6][26][6] = gma2*S[2][26][6]+     fak * (S[2][11][6] + S[2][26][2] );
     S[8][26][7] = gma0*S[2][26][7]+     fak * (S[1][26][7] + 3.e0*S[2][16][7] );
     S[5][26][7] = gma1*S[2][26][7]+     fak * S[2][26][4];
     S[6][26][7] = gma2*S[2][26][7]+     fak * (S[2][11][7] + S[2][26][3] );
     S[8][26][8] = gma0*S[2][26][8]+     fak * (S[1][26][8] + 3.e0*S[2][16][8] + 2.e0*S[2][26][2] );
     S[5][26][8] = gma1*S[2][26][8];
     S[6][26][8] = gma2*S[2][26][8]+     fak * S[2][11][8];
     S[8][26][9] = gma0*S[2][26][9]+     fak * (S[1][26][9] + 3.e0*S[2][16][9] );
     S[5][26][9] = gma1*S[2][26][9]+     fak2* S[2][26][3];
     S[6][26][9] = gma2*S[2][26][9]+     fak * S[2][11][9];
     S[8][26][10] = gma0*S[2][26][10]+     fak * (S[1][26][10] + 3.e0*S[2][16][10] );
     S[5][26][10] = gma1*S[2][26][10];
     S[6][26][10] = gma2*S[2][26][10]+     fak * (S[2][11][10] + 2.e0*S[2][26][4] );
     S[8][27][5] = gma0*S[2][27][5]+     fak * (S[1][27][5] + S[2][13][5] + S[2][27][3] );
     S[5][27][5] = gma1*S[2][27][5]+     fak * S[2][27][2];
     S[6][27][5] = gma2*S[2][27][5]+     fak3* S[2][17][5];
     S[8][27][6] = gma0*S[2][27][6]+     fak * (S[1][27][6] + S[2][13][6] + S[2][27][4] );
     S[5][27][6] = gma1*S[2][27][6];
     S[6][27][6] = gma2*S[2][27][6]+     fak * (3.e0*S[2][17][6] + S[2][27][2] );
     S[8][27][7] = gma0*S[2][27][7]+     fak * (S[1][27][7] + S[2][13][7] );
     S[5][27][7] = gma1*S[2][27][7]+     fak * S[2][27][4];
     S[6][27][7] = gma2*S[2][27][7]+     fak * (3.e0*S[2][17][7] + S[2][27][3] );
     S[8][27][8] = gma0*S[2][27][8]+     fak * (S[1][27][8] + S[2][13][8] + 2.e0*S[2][27][2] );
     S[5][27][8] = gma1*S[2][27][8];
     S[6][27][8] = gma2*S[2][27][8]+     fak3* S[2][17][8];
     S[8][27][9] = gma0*S[2][27][9]+     fak * (S[1][27][9] + S[2][13][9] );
     S[5][27][9] = gma1*S[2][27][9]+     fak2* S[2][27][3];
     S[6][27][9] = gma2*S[2][27][9]+     fak3* S[2][17][9];
     S[8][27][10] = gma0*S[2][27][10]+     fak * (S[1][27][10] + S[2][13][10] );
     S[5][27][10] = gma1*S[2][27][10];
     S[6][27][10] = gma2*S[2][27][10]+     fak * (3.e0*S[2][17][10] + 2.e0*S[2][27][4] );
     S[8][28][5] = gma0*S[2][28][5]+     fak * (S[1][28][5] + S[2][28][3] );
     S[5][28][5] = gma1*S[2][28][5]+     fak * (3.e0*S[2][18][5] + S[2][28][2] );
     S[6][28][5] = gma2*S[2][28][5]+     fak * S[2][12][5];
     S[8][28][6] = gma0*S[2][28][6]+     fak * (S[1][28][6] + S[2][28][4] );
     S[5][28][6] = gma1*S[2][28][6]+     fak3* S[2][18][6];
     S[6][28][6] = gma2*S[2][28][6]+     fak * (S[2][12][6] + S[2][28][2] );
     S[8][28][7] = gma0*S[2][28][7]+     fak * S[1][28][7];
     S[5][28][7] = gma1*S[2][28][7]+     fak * (3.e0*S[2][18][7] + S[2][28][4] );
     S[6][28][7] = gma2*S[2][28][7]+     fak * (S[2][12][7] + S[2][28][3] );
     S[8][28][8] = gma0*S[2][28][8]+     fak * (S[1][28][8] + 2.e0*S[2][28][2] );
     S[5][28][8] = gma1*S[2][28][8]+     fak3* S[2][18][8];
     S[6][28][8] = gma2*S[2][28][8]+     fak * S[2][12][8];
     S[8][28][9] = gma0*S[2][28][9]+     fak * S[1][28][9];
     S[5][28][9] = gma1*S[2][28][9]+     fak * (3.e0*S[2][18][9] + 2.e0*S[2][28][3] );
     S[6][28][9] = gma2*S[2][28][9]+     fak * S[2][12][9];
     S[8][28][10] = gma0*S[2][28][10]+     fak * S[1][28][10];
     S[5][28][10] = gma1*S[2][28][10]+     fak3* S[2][18][10];
     S[6][28][10] = gma2*S[2][28][10]+     fak * (S[2][12][10] + 2.e0*S[2][28][4] );
     S[8][29][5] = gma0*S[2][29][5]+     fak * (S[1][29][5] + S[2][29][3] );
     S[5][29][5] = gma1*S[2][29][5]+     fak * (S[2][13][5] + S[2][29][2] );
     S[6][29][5] = gma2*S[2][29][5]+     fak3* S[2][19][5];
     S[8][29][6] = gma0*S[2][29][6]+     fak * (S[1][29][6] + S[2][29][4] );
     S[5][29][6] = gma1*S[2][29][6]+     fak * S[2][13][6];
     S[6][29][6] = gma2*S[2][29][6]+     fak * (3.e0*S[2][19][6] + S[2][29][2] );
     S[8][29][7] = gma0*S[2][29][7]+     fak * S[1][29][7];
     S[5][29][7] = gma1*S[2][29][7]+     fak * (S[2][13][7] + S[2][29][4] );
     S[6][29][7] = gma2*S[2][29][7]+     fak * (3.e0*S[2][19][7] + S[2][29][3] );
     S[8][29][8] = gma0*S[2][29][8]+     fak * (S[1][29][8] + 2.e0*S[2][29][2] );
     S[5][29][8] = gma1*S[2][29][8]+     fak * S[2][13][8];
     S[6][29][8] = gma2*S[2][29][8]+     fak3* S[2][19][8];
     S[8][29][9] = gma0*S[2][29][9]+     fak * S[1][29][9];
     S[5][29][9] = gma1*S[2][29][9]+     fak * (S[2][13][9] + 2.e0*S[2][29][3] );
     S[6][29][9] = gma2*S[2][29][9]+     fak3* S[2][19][9];
     S[8][29][10] = gma0*S[2][29][10]+     fak * S[1][29][10];
     S[5][29][10] = gma1*S[2][29][10]+     fak * S[2][13][10];
     S[6][29][10] = gma2*S[2][29][10]+     fak * (3.e0*S[2][19][10] + 2.e0*S[2][29][4] );
     S[8][30][5] = gma0*S[2][30][5]+     fak * (S[1][30][5] + 2.e0*S[2][17][5] + S[2][30][3] );
     S[5][30][5] = gma1*S[2][30][5]+     fak * S[2][30][2];
     S[6][30][5] = gma2*S[2][30][5]+     fak2* S[2][16][5];
     S[8][30][6] = gma0*S[2][30][6]+     fak * (S[1][30][6] + 2.e0*S[2][17][6] + S[2][30][4] );
     S[5][30][6] = gma1*S[2][30][6];
     S[6][30][6] = gma2*S[2][30][6]+     fak * (2.e0*S[2][16][6] + S[2][30][2] );
     S[8][30][7] = gma0*S[2][30][7]+     fak * (S[1][30][7] + 2.e0*S[2][17][7] );
     S[5][30][7] = gma1*S[2][30][7]+     fak * S[2][30][4];
     S[6][30][7] = gma2*S[2][30][7]+     fak * (2.e0*S[2][16][7] + S[2][30][3] );
     S[8][30][8] = gma0*S[2][30][8]+     fak * (S[1][30][8] + 2.e0*S[2][17][8] + 2.e0*S[2][30][2] );
     S[5][30][8] = gma1*S[2][30][8];
     S[6][30][8] = gma2*S[2][30][8]+     fak2* S[2][16][8];
     S[8][30][9] = gma0*S[2][30][9]+     fak * (S[1][30][9] + 2.e0*S[2][17][9] );
     S[5][30][9] = gma1*S[2][30][9]+     fak2* S[2][30][3];
     S[6][30][9] = gma2*S[2][30][9]+     fak2* S[2][16][9];
     S[8][30][10] = gma0*S[2][30][10]+     fak * (S[1][30][10] + 2.e0*S[2][17][10] );
     S[5][30][10] = gma1*S[2][30][10];
     S[6][30][10] = gma2*S[2][30][10]+     fak2* (S[2][16][10] + S[2][30][4] );
     S[8][31][5] = gma0*S[2][31][5]+     fak * (S[1][31][5] + S[2][31][3] );
     S[5][31][5] = gma1*S[2][31][5]+     fak * (2.e0*S[2][19][5] + S[2][31][2] );
     S[6][31][5] = gma2*S[2][31][5]+     fak2* S[2][18][5];
     S[8][31][6] = gma0*S[2][31][6]+     fak * (S[1][31][6] + S[2][31][4] );
     S[5][31][6] = gma1*S[2][31][6]+     fak2* S[2][19][6];
     S[6][31][6] = gma2*S[2][31][6]+     fak * (2.e0*S[2][18][6] + S[2][31][2] );
     S[8][31][7] = gma0*S[2][31][7]+     fak * S[1][31][7];
     S[5][31][7] = gma1*S[2][31][7]+     fak * (2.e0*S[2][19][7] + S[2][31][4] );
     S[6][31][7] = gma2*S[2][31][7]+     fak * (2.e0*S[2][18][7] + S[2][31][3] );
     S[8][31][8] = gma0*S[2][31][8]+     fak * (S[1][31][8] + 2.e0*S[2][31][2] );
     S[5][31][8] = gma1*S[2][31][8]+     fak2* S[2][19][8];
     S[6][31][8] = gma2*S[2][31][8]+     fak2* S[2][18][8];
     S[8][31][9] = gma0*S[2][31][9]+     fak * S[1][31][9];
     S[5][31][9] = gma1*S[2][31][9]+     fak2* (S[2][19][9] + S[2][31][3] );
     S[6][31][9] = gma2*S[2][31][9]+     fak2* S[2][18][9];
     S[8][31][10] = gma0*S[2][31][10]+     fak * S[1][31][10];
     S[5][31][10] = gma1*S[2][31][10]+     fak2* S[2][19][10];
     S[6][31][10] = gma2*S[2][31][10]+     fak2* (S[2][18][10] + S[2][31][4] );
     S[8][32][5] = gma0*S[2][32][5]+     fak * (S[1][32][5] + 2.e0*S[2][15][5] + S[2][32][3] );
     S[5][32][5] = gma1*S[2][32][5]+     fak * (2.e0*S[2][14][5] + S[2][32][2] );
     S[6][32][5] = gma2*S[2][32][5];
     S[8][32][6] = gma0*S[2][32][6]+     fak * (S[1][32][6] + 2.e0*S[2][15][6] + S[2][32][4] );
     S[5][32][6] = gma1*S[2][32][6]+     fak2* S[2][14][6];
     S[6][32][6] = gma2*S[2][32][6]+     fak * S[2][32][2];
     S[8][32][7] = gma0*S[2][32][7]+     fak * (S[1][32][7] + 2.e0*S[2][15][7] );
     S[5][32][7] = gma1*S[2][32][7]+     fak * (2.e0*S[2][14][7] + S[2][32][4] );
     S[6][32][7] = gma2*S[2][32][7]+     fak * S[2][32][3];
     S[8][32][8] = gma0*S[2][32][8]+     fak * (S[1][32][8] + 2.e0*S[2][15][8] + 2.e0*S[2][32][2] );
     S[5][32][8] = gma1*S[2][32][8]+     fak2* S[2][14][8];
     S[6][32][8] = gma2*S[2][32][8];
     S[8][32][9] = gma0*S[2][32][9]+     fak * (S[1][32][9] + 2.e0*S[2][15][9] );
     S[5][32][9] = gma1*S[2][32][9]+     fak2* (S[2][14][9] + S[2][32][3] );
     S[6][32][9] = gma2*S[2][32][9];
     S[8][32][10] = gma0*S[2][32][10]+     fak * (S[1][32][10] + 2.e0*S[2][15][10] );
     S[5][32][10] = gma1*S[2][32][10]+     fak2* S[2][14][10];
     S[6][32][10] = gma2*S[2][32][10]+     fak2* S[2][32][4];
     S[8][33][5] = gma0*S[2][33][5]+     fak * (S[1][33][5] + 2.e0*S[2][20][5] + S[2][33][3] );
     S[5][33][5] = gma1*S[2][33][5]+     fak * (S[2][16][5] + S[2][33][2] );
     S[6][33][5] = gma2*S[2][33][5]+     fak * S[2][14][5];
     S[8][33][6] = gma0*S[2][33][6]+     fak * (S[1][33][6] + 2.e0*S[2][20][6] + S[2][33][4] );
     S[5][33][6] = gma1*S[2][33][6]+     fak * S[2][16][6];
     S[6][33][6] = gma2*S[2][33][6]+     fak * (S[2][14][6] + S[2][33][2] );
     S[8][33][7] = gma0*S[2][33][7]+     fak * (S[1][33][7] + 2.e0*S[2][20][7] );
     S[5][33][7] = gma1*S[2][33][7]+     fak * (S[2][16][7] + S[2][33][4] );
     S[6][33][7] = gma2*S[2][33][7]+     fak * (S[2][14][7] + S[2][33][3] );
     S[8][33][8] = gma0*S[2][33][8]+     fak * (S[1][33][8] + 2.e0*S[2][20][8] + 2.e0*S[2][33][2] );
     S[5][33][8] = gma1*S[2][33][8]+     fak * S[2][16][8];
     S[6][33][8] = gma2*S[2][33][8]+     fak * S[2][14][8];
     S[8][33][9] = gma0*S[2][33][9]+     fak * (S[1][33][9] + 2.e0*S[2][20][9] );
     S[5][33][9] = gma1*S[2][33][9]+     fak * (S[2][16][9] + 2.e0*S[2][33][3] );
     S[6][33][9] = gma2*S[2][33][9]+     fak * S[2][14][9];
     S[8][33][10] = gma0*S[2][33][10]+     fak * (S[1][33][10] + 2.e0*S[2][20][10] );
     S[5][33][10] = gma1*S[2][33][10]+     fak * S[2][16][10];
     S[6][33][10] = gma2*S[2][33][10]+     fak * (S[2][14][10] + 2.e0*S[2][33][4] );
     S[8][34][5] = gma0*S[2][34][5]+     fak * (S[1][34][5] + S[2][18][5] + S[2][34][3] );
     S[5][34][5] = gma1*S[2][34][5]+     fak * (2.e0*S[2][20][5] + S[2][34][2] );
     S[6][34][5] = gma2*S[2][34][5]+     fak * S[2][15][5];
     S[8][34][6] = gma0*S[2][34][6]+     fak * (S[1][34][6] + S[2][18][6] + S[2][34][4] );
     S[5][34][6] = gma1*S[2][34][6]+     fak2* S[2][20][6];
     S[6][34][6] = gma2*S[2][34][6]+     fak * (S[2][15][6] + S[2][34][2] );
     S[8][34][7] = gma0*S[2][34][7]+     fak * (S[1][34][7] + S[2][18][7] );
     S[5][34][7] = gma1*S[2][34][7]+     fak * (2.e0*S[2][20][7] + S[2][34][4] );
     S[6][34][7] = gma2*S[2][34][7]+     fak * (S[2][15][7] + S[2][34][3] );
     S[8][34][8] = gma0*S[2][34][8]+     fak * (S[1][34][8] + S[2][18][8] + 2.e0*S[2][34][2] );
     S[5][34][8] = gma1*S[2][34][8]+     fak2* S[2][20][8];
     S[6][34][8] = gma2*S[2][34][8]+     fak * S[2][15][8];
     S[8][34][9] = gma0*S[2][34][9]+     fak * (S[1][34][9] + S[2][18][9] );
     S[5][34][9] = gma1*S[2][34][9]+     fak2* (S[2][20][9] + S[2][34][3] );
     S[6][34][9] = gma2*S[2][34][9]+     fak * S[2][15][9];
     S[8][34][10] = gma0*S[2][34][10]+     fak * (S[1][34][10] + S[2][18][10] );
     S[5][34][10] = gma1*S[2][34][10]+     fak2* S[2][20][10];
     S[6][34][10] = gma2*S[2][34][10]+     fak * (S[2][15][10] + 2.e0*S[2][34][4] );
     S[8][35][5] = gma0*S[2][35][5]+     fak * (S[1][35][5] + S[2][19][5] + S[2][35][3] );
     S[5][35][5] = gma1*S[2][35][5]+     fak * (S[2][17][5] + S[2][35][2] );
     S[6][35][5] = gma2*S[2][35][5]+     fak2* S[2][20][5];
     S[8][35][6] = gma0*S[2][35][6]+     fak * (S[1][35][6] + S[2][19][6] + S[2][35][4] );
     S[5][35][6] = gma1*S[2][35][6]+     fak * S[2][17][6];
     S[6][35][6] = gma2*S[2][35][6]+     fak * (2.e0*S[2][20][6] + S[2][35][2] );
     S[8][35][7] = gma0*S[2][35][7]+     fak * (S[1][35][7] + S[2][19][7] );
     S[5][35][7] = gma1*S[2][35][7]+     fak * (S[2][17][7] + S[2][35][4] );
     S[6][35][7] = gma2*S[2][35][7]+     fak * (2.e0*S[2][20][7] + S[2][35][3] );
     S[8][35][8] = gma0*S[2][35][8]+     fak * (S[1][35][8] + S[2][19][8] + 2.e0*S[2][35][2] );
     S[5][35][8] = gma1*S[2][35][8]+     fak * S[2][17][8];
     S[6][35][8] = gma2*S[2][35][8]+     fak2* S[2][20][8];
     S[8][35][9] = gma0*S[2][35][9]+     fak * (S[1][35][9] + S[2][19][9] );
     S[5][35][9] = gma1*S[2][35][9]+     fak * (S[2][17][9] + 2.e0*S[2][35][3] );
     S[6][35][9] = gma2*S[2][35][9]+     fak2* S[2][20][9];
     S[8][35][10] = gma0*S[2][35][10]+     fak * (S[1][35][10] + S[2][19][10] );
     S[5][35][10] = gma1*S[2][35][10]+     fak * S[2][17][10];
     S[6][35][10] = gma2*S[2][35][10]+     fak2* (S[2][20][10] + S[2][35][4] );
     S[9][21][5] = gma1*S[3][21][5]+     fak * (S[1][21][5] + S[3][21][2] );
     S[7][21][5] = gma2*S[3][21][5];
     S[9][21][6] = gma1*S[3][21][6]+     fak * S[1][21][6];
     S[7][21][6] = gma2*S[3][21][6]+     fak * S[3][21][2];
     S[9][21][7] = gma1*S[3][21][7]+     fak * (S[1][21][7] + S[3][21][4] );
     S[7][21][7] = gma2*S[3][21][7]+     fak * S[3][21][3];
     S[9][21][8] = gma1*S[3][21][8]+     fak * S[1][21][8];
     S[7][21][8] = gma2*S[3][21][8];
     S[9][21][9] = gma1*S[3][21][9]+     fak * (S[1][21][9] + 2.e0*S[3][21][3] );
     S[7][21][9] = gma2*S[3][21][9];
     S[9][21][10] = gma1*S[3][21][10]+     fak * S[1][21][10];
     S[7][21][10] = gma2*S[3][21][10]+     fak2* S[3][21][4];
     S[9][22][5] = gma1*S[3][22][5]+     fak * (S[1][22][5] + 4.e0*S[3][12][5] + S[3][22][2] );
     S[7][22][5] = gma2*S[3][22][5];
     S[9][22][6] = gma1*S[3][22][6]+     fak * (S[1][22][6] + 4.e0*S[3][12][6] );
     S[7][22][6] = gma2*S[3][22][6]+     fak * S[3][22][2];
     S[9][22][7] = gma1*S[3][22][7]+     fak * (S[1][22][7] + 4.e0*S[3][12][7] + S[3][22][4] );
     S[7][22][7] = gma2*S[3][22][7]+     fak * S[3][22][3];
     S[9][22][8] = gma1*S[3][22][8]+     fak * (S[1][22][8] + 4.e0*S[3][12][8] );
     S[7][22][8] = gma2*S[3][22][8];
     S[9][22][9] = gma1*S[3][22][9]+     fak * (S[1][22][9] + 4.e0*S[3][12][9] + 2.e0*S[3][22][3] );
     S[7][22][9] = gma2*S[3][22][9];
     S[9][22][10] = gma1*S[3][22][10]+     fak * (S[1][22][10] + 4.e0*S[3][12][10] );
     S[7][22][10] = gma2*S[3][22][10]+     fak2* S[3][22][4];
     S[9][23][5] = gma1*S[3][23][5]+     fak * (S[1][23][5] + S[3][23][2] );
     S[7][23][5] = gma2*S[3][23][5]+     fak4* S[3][13][5];
     S[9][23][6] = gma1*S[3][23][6]+     fak * S[1][23][6];
     S[7][23][6] = gma2*S[3][23][6]+     fak * (4.e0*S[3][13][6] + S[3][23][2] );
     S[9][23][7] = gma1*S[3][23][7]+     fak * (S[1][23][7] + S[3][23][4] );
     S[7][23][7] = gma2*S[3][23][7]+     fak * (4.e0*S[3][13][7] + S[3][23][3] );
     S[9][23][8] = gma1*S[3][23][8]+     fak * S[1][23][8];
     S[7][23][8] = gma2*S[3][23][8]+     fak4* S[3][13][8];
     S[9][23][9] = gma1*S[3][23][9]+     fak * (S[1][23][9] + 2.e0*S[3][23][3] );
     S[7][23][9] = gma2*S[3][23][9]+     fak4* S[3][13][9];
     S[9][23][10] = gma1*S[3][23][10]+     fak * S[1][23][10];
     S[7][23][10] = gma2*S[3][23][10]+     fak2* (2.e0*S[3][13][10] + S[3][23][4] );
     S[9][24][5] = gma1*S[3][24][5]+     fak * (S[1][24][5] + S[3][11][5] + S[3][24][2] );
     S[7][24][5] = gma2*S[3][24][5];
     S[9][24][6] = gma1*S[3][24][6]+     fak * (S[1][24][6] + S[3][11][6] );
     S[7][24][6] = gma2*S[3][24][6]+     fak * S[3][24][2];
     S[9][24][7] = gma1*S[3][24][7]+     fak * (S[1][24][7] + S[3][11][7] + S[3][24][4] );
     S[7][24][7] = gma2*S[3][24][7]+     fak * S[3][24][3];
     S[9][24][8] = gma1*S[3][24][8]+     fak * (S[1][24][8] + S[3][11][8] );
     S[7][24][8] = gma2*S[3][24][8];
     S[9][24][9] = gma1*S[3][24][9]+     fak * (S[1][24][9] + S[3][11][9] + 2.e0*S[3][24][3] );
     S[7][24][9] = gma2*S[3][24][9];
     S[9][24][10] = gma1*S[3][24][10]+     fak * (S[1][24][10] + S[3][11][10] );
     S[7][24][10] = gma2*S[3][24][10]+     fak2* S[3][24][4];
     S[9][25][5] = gma1*S[3][25][5]+     fak * (S[1][25][5] + 3.e0*S[3][15][5] + S[3][25][2] );
     S[7][25][5] = gma2*S[3][25][5];
     S[9][25][6] = gma1*S[3][25][6]+     fak * (S[1][25][6] + 3.e0*S[3][15][6] );
     S[7][25][6] = gma2*S[3][25][6]+     fak * S[3][25][2];
     S[9][25][7] = gma1*S[3][25][7]+     fak * (S[1][25][7] + 3.e0*S[3][15][7] + S[3][25][4] );
     S[7][25][7] = gma2*S[3][25][7]+     fak * S[3][25][3];
     S[9][25][8] = gma1*S[3][25][8]+     fak * (S[1][25][8] + 3.e0*S[3][15][8] );
     S[7][25][8] = gma2*S[3][25][8];
     S[9][25][9] = gma1*S[3][25][9]+     fak * (S[1][25][9] + 3.e0*S[3][15][9] + 2.e0*S[3][25][3] );
     S[7][25][9] = gma2*S[3][25][9];
     S[9][25][10] = gma1*S[3][25][10]+     fak * (S[1][25][10] + 3.e0*S[3][15][10] );
     S[7][25][10] = gma2*S[3][25][10]+     fak2* S[3][25][4];
     S[9][26][5] = gma1*S[3][26][5]+     fak * (S[1][26][5] + S[3][26][2] );
     S[7][26][5] = gma2*S[3][26][5]+     fak * S[3][11][5];
     S[9][26][6] = gma1*S[3][26][6]+     fak * S[1][26][6];
     S[7][26][6] = gma2*S[3][26][6]+     fak * (S[3][11][6] + S[3][26][2] );
     S[9][26][7] = gma1*S[3][26][7]+     fak * (S[1][26][7] + S[3][26][4] );
     S[7][26][7] = gma2*S[3][26][7]+     fak * (S[3][11][7] + S[3][26][3] );
     S[9][26][8] = gma1*S[3][26][8]+     fak * S[1][26][8];
     S[7][26][8] = gma2*S[3][26][8]+     fak * S[3][11][8];
     S[9][26][9] = gma1*S[3][26][9]+     fak * (S[1][26][9] + 2.e0*S[3][26][3] );
     S[7][26][9] = gma2*S[3][26][9]+     fak * S[3][11][9];
     S[9][26][10] = gma1*S[3][26][10]+     fak * S[1][26][10];
     S[7][26][10] = gma2*S[3][26][10]+     fak * (S[3][11][10] + 2.e0*S[3][26][4] );
     S[9][27][5] = gma1*S[3][27][5]+     fak * (S[1][27][5] + S[3][27][2] );
     S[7][27][5] = gma2*S[3][27][5]+     fak3* S[3][17][5];
     S[9][27][6] = gma1*S[3][27][6]+     fak * S[1][27][6];
     S[7][27][6] = gma2*S[3][27][6]+     fak * (3.e0*S[3][17][6] + S[3][27][2] );
     S[9][27][7] = gma1*S[3][27][7]+     fak * (S[1][27][7] + S[3][27][4] );
     S[7][27][7] = gma2*S[3][27][7]+     fak * (3.e0*S[3][17][7] + S[3][27][3] );
     S[9][27][8] = gma1*S[3][27][8]+     fak * S[1][27][8];
     S[7][27][8] = gma2*S[3][27][8]+     fak3* S[3][17][8];
     S[9][27][9] = gma1*S[3][27][9]+     fak * (S[1][27][9] + 2.e0*S[3][27][3] );
     S[7][27][9] = gma2*S[3][27][9]+     fak3* S[3][17][9];
     S[9][27][10] = gma1*S[3][27][10]+     fak * S[1][27][10];
     S[7][27][10] = gma2*S[3][27][10]+     fak * (3.e0*S[3][17][10] + 2.e0*S[3][27][4] );
     S[9][28][5] = gma1*S[3][28][5]+     fak * (S[1][28][5] + 3.e0*S[3][18][5] + S[3][28][2] );
     S[7][28][5] = gma2*S[3][28][5]+     fak * S[3][12][5];
     S[9][28][6] = gma1*S[3][28][6]+     fak * (S[1][28][6] + 3.e0*S[3][18][6] );
     S[7][28][6] = gma2*S[3][28][6]+     fak * (S[3][12][6] + S[3][28][2] );
     S[9][28][7] = gma1*S[3][28][7]+     fak * (S[1][28][7] + 3.e0*S[3][18][7] + S[3][28][4] );
     S[7][28][7] = gma2*S[3][28][7]+     fak * (S[3][12][7] + S[3][28][3] );
     S[9][28][8] = gma1*S[3][28][8]+     fak * (S[1][28][8] + 3.e0*S[3][18][8] );
     S[7][28][8] = gma2*S[3][28][8]+     fak * S[3][12][8];
     S[9][28][9] = gma1*S[3][28][9]+     fak * (S[1][28][9] + 3.e0*S[3][18][9] + 2.e0*S[3][28][3] );
     S[7][28][9] = gma2*S[3][28][9]+     fak * S[3][12][9];
     S[9][28][10] = gma1*S[3][28][10]+     fak * (S[1][28][10] + 3.e0*S[3][18][10] );
     S[7][28][10] = gma2*S[3][28][10]+     fak * (S[3][12][10] + 2.e0*S[3][28][4] );
     S[9][29][5] = gma1*S[3][29][5]+     fak * (S[1][29][5] + S[3][13][5] + S[3][29][2] );
     S[7][29][5] = gma2*S[3][29][5]+     fak3* S[3][19][5];
     S[9][29][6] = gma1*S[3][29][6]+     fak * (S[1][29][6] + S[3][13][6] );
     S[7][29][6] = gma2*S[3][29][6]+     fak * (3.e0*S[3][19][6] + S[3][29][2] );
     S[9][29][7] = gma1*S[3][29][7]+     fak * (S[1][29][7] + S[3][13][7] + S[3][29][4] );
     S[7][29][7] = gma2*S[3][29][7]+     fak * (3.e0*S[3][19][7] + S[3][29][3] );
     S[9][29][8] = gma1*S[3][29][8]+     fak * (S[1][29][8] + S[3][13][8] );
     S[7][29][8] = gma2*S[3][29][8]+     fak3* S[3][19][8];
     S[9][29][9] = gma1*S[3][29][9]+     fak * (S[1][29][9] + S[3][13][9] + 2.e0*S[3][29][3] );
     S[7][29][9] = gma2*S[3][29][9]+     fak3* S[3][19][9];
     S[9][29][10] = gma1*S[3][29][10]+     fak * (S[1][29][10] + S[3][13][10] );
     S[7][29][10] = gma2*S[3][29][10]+     fak * (3.e0*S[3][19][10] + 2.e0*S[3][29][4] );
     S[9][30][5] = gma1*S[3][30][5]+     fak * (S[1][30][5] + S[3][30][2] );
     S[7][30][5] = gma2*S[3][30][5]+     fak2* S[3][16][5];
     S[9][30][6] = gma1*S[3][30][6]+     fak * S[1][30][6];
     S[7][30][6] = gma2*S[3][30][6]+     fak * (2.e0*S[3][16][6] + S[3][30][2] );
     S[9][30][7] = gma1*S[3][30][7]+     fak * (S[1][30][7] + S[3][30][4] );
     S[7][30][7] = gma2*S[3][30][7]+     fak * (2.e0*S[3][16][7] + S[3][30][3] );
     S[9][30][8] = gma1*S[3][30][8]+     fak * S[1][30][8];
     S[7][30][8] = gma2*S[3][30][8]+     fak2* S[3][16][8];
     S[9][30][9] = gma1*S[3][30][9]+     fak * (S[1][30][9] + 2.e0*S[3][30][3] );
     S[7][30][9] = gma2*S[3][30][9]+     fak2* S[3][16][9];
     S[9][30][10] = gma1*S[3][30][10]+     fak * S[1][30][10];
     S[7][30][10] = gma2*S[3][30][10]+     fak2* (S[3][16][10] + S[3][30][4] );
     S[9][31][5] = gma1*S[3][31][5]+     fak * (S[1][31][5] + 2.e0*S[3][19][5] + S[3][31][2] );
     S[7][31][5] = gma2*S[3][31][5]+     fak2* S[3][18][5];
     S[9][31][6] = gma1*S[3][31][6]+     fak * (S[1][31][6] + 2.e0*S[3][19][6] );
     S[7][31][6] = gma2*S[3][31][6]+     fak * (2.e0*S[3][18][6] + S[3][31][2] );
     S[9][31][7] = gma1*S[3][31][7]+     fak * (S[1][31][7] + 2.e0*S[3][19][7] + S[3][31][4] );
     S[7][31][7] = gma2*S[3][31][7]+     fak * (2.e0*S[3][18][7] + S[3][31][3] );
     S[9][31][8] = gma1*S[3][31][8]+     fak * (S[1][31][8] + 2.e0*S[3][19][8] );
     S[7][31][8] = gma2*S[3][31][8]+     fak2* S[3][18][8];
     S[9][31][9] = gma1*S[3][31][9]+     fak * (S[1][31][9] + 2.e0*S[3][19][9] + 2.e0*S[3][31][3] );
     S[7][31][9] = gma2*S[3][31][9]+     fak2* S[3][18][9];
     S[9][31][10] = gma1*S[3][31][10]+     fak * (S[1][31][10] + 2.e0*S[3][19][10] );
     S[7][31][10] = gma2*S[3][31][10]+     fak2* (S[3][18][10] + S[3][31][4] );
     S[9][32][5] = gma1*S[3][32][5]+     fak * (S[1][32][5] + 2.e0*S[3][14][5] + S[3][32][2] );
     S[7][32][5] = gma2*S[3][32][5];
     S[9][32][6] = gma1*S[3][32][6]+     fak * (S[1][32][6] + 2.e0*S[3][14][6] );
     S[7][32][6] = gma2*S[3][32][6]+     fak * S[3][32][2];
     S[9][32][7] = gma1*S[3][32][7]+     fak * (S[1][32][7] + 2.e0*S[3][14][7] + S[3][32][4] );
     S[7][32][7] = gma2*S[3][32][7]+     fak * S[3][32][3];
     S[9][32][8] = gma1*S[3][32][8]+     fak * (S[1][32][8] + 2.e0*S[3][14][8] );
     S[7][32][8] = gma2*S[3][32][8];
     S[9][32][9] = gma1*S[3][32][9]+     fak * (S[1][32][9] + 2.e0*S[3][14][9] + 2.e0*S[3][32][3] );
     S[7][32][9] = gma2*S[3][32][9];
     S[9][32][10] = gma1*S[3][32][10]+     fak * (S[1][32][10] + 2.e0*S[3][14][10] );
     S[7][32][10] = gma2*S[3][32][10]+     fak2* S[3][32][4];
     S[9][33][5] = gma1*S[3][33][5]+     fak * (S[1][33][5] + S[3][16][5] + S[3][33][2] );
     S[7][33][5] = gma2*S[3][33][5]+     fak * S[3][14][5];
     S[9][33][6] = gma1*S[3][33][6]+     fak * (S[1][33][6] + S[3][16][6] );
     S[7][33][6] = gma2*S[3][33][6]+     fak * (S[3][14][6] + S[3][33][2] );
     S[9][33][7] = gma1*S[3][33][7]+     fak * (S[1][33][7] + S[3][16][7] + S[3][33][4] );
     S[7][33][7] = gma2*S[3][33][7]+     fak * (S[3][14][7] + S[3][33][3] );
     S[9][33][8] = gma1*S[3][33][8]+     fak * (S[1][33][8] + S[3][16][8] );
     S[7][33][8] = gma2*S[3][33][8]+     fak * S[3][14][8];
     S[9][33][9] = gma1*S[3][33][9]+     fak * (S[1][33][9] + S[3][16][9] + 2.e0*S[3][33][3] );
     S[7][33][9] = gma2*S[3][33][9]+     fak * S[3][14][9];
     S[9][33][10] = gma1*S[3][33][10]+     fak * (S[1][33][10] + S[3][16][10] );
     S[7][33][10] = gma2*S[3][33][10]+     fak * (S[3][14][10] + 2.e0*S[3][33][4] );
     S[9][34][5] = gma1*S[3][34][5]+     fak * (S[1][34][5] + 2.e0*S[3][20][5] + S[3][34][2] );
     S[7][34][5] = gma2*S[3][34][5]+     fak * S[3][15][5];
     S[9][34][6] = gma1*S[3][34][6]+     fak * (S[1][34][6] + 2.e0*S[3][20][6] );
     S[7][34][6] = gma2*S[3][34][6]+     fak * (S[3][15][6] + S[3][34][2] );
     S[9][34][7] = gma1*S[3][34][7]+     fak * (S[1][34][7] + 2.e0*S[3][20][7] + S[3][34][4] );
     S[7][34][7] = gma2*S[3][34][7]+     fak * (S[3][15][7] + S[3][34][3] );
     S[9][34][8] = gma1*S[3][34][8]+     fak * (S[1][34][8] + 2.e0*S[3][20][8] );
     S[7][34][8] = gma2*S[3][34][8]+     fak * S[3][15][8];
     S[9][34][9] = gma1*S[3][34][9]+     fak * (S[1][34][9] + 2.e0*S[3][20][9] + 2.e0*S[3][34][3] );
     S[7][34][9] = gma2*S[3][34][9]+     fak * S[3][15][9];
     S[9][34][10] = gma1*S[3][34][10]+     fak * (S[1][34][10] + 2.e0*S[3][20][10] );
     S[7][34][10] = gma2*S[3][34][10]+     fak * (S[3][15][10] + 2.e0*S[3][34][4] );
     S[9][35][5] = gma1*S[3][35][5]+     fak * (S[1][35][5] + S[3][17][5] + S[3][35][2] );
     S[7][35][5] = gma2*S[3][35][5]+     fak2* S[3][20][5];
     S[9][35][6] = gma1*S[3][35][6]+     fak * (S[1][35][6] + S[3][17][6] );
     S[7][35][6] = gma2*S[3][35][6]+     fak * (2.e0*S[3][20][6] + S[3][35][2] );
     S[9][35][7] = gma1*S[3][35][7]+     fak * (S[1][35][7] + S[3][17][7] + S[3][35][4] );
     S[7][35][7] = gma2*S[3][35][7]+     fak * (2.e0*S[3][20][7] + S[3][35][3] );
     S[9][35][8] = gma1*S[3][35][8]+     fak * (S[1][35][8] + S[3][17][8] );
     S[7][35][8] = gma2*S[3][35][8]+     fak2* S[3][20][8];
     S[9][35][9] = gma1*S[3][35][9]+     fak * (S[1][35][9] + S[3][17][9] + 2.e0*S[3][35][3] );
     S[7][35][9] = gma2*S[3][35][9]+     fak2* S[3][20][9];
     S[9][35][10] = gma1*S[3][35][10]+     fak * (S[1][35][10] + S[3][17][10] );
     S[7][35][10] = gma2*S[3][35][10]+     fak2* (S[3][20][10] + S[3][35][4] );
     S[10][21][5] = gma2*S[4][21][5]+     fak * S[1][21][5];
     S[10][21][6] = gma2*S[4][21][6]+     fak * (S[1][21][6] + S[4][21][2] );
     S[10][21][7] = gma2*S[4][21][7]+     fak * (S[1][21][7] + S[4][21][3] );
     S[10][21][8] = gma2*S[4][21][8]+     fak * S[1][21][8];
     S[10][21][9] = gma2*S[4][21][9]+     fak * S[1][21][9];
     S[10][21][10] = gma2*S[4][21][10]+     fak * (S[1][21][10] + 2.e0*S[4][21][4] );
     S[10][22][5] = gma2*S[4][22][5]+     fak * S[1][22][5];
     S[10][22][6] = gma2*S[4][22][6]+     fak * (S[1][22][6] + S[4][22][2] );
     S[10][22][7] = gma2*S[4][22][7]+     fak * (S[1][22][7] + S[4][22][3] );
     S[10][22][8] = gma2*S[4][22][8]+     fak * S[1][22][8];
     S[10][22][9] = gma2*S[4][22][9]+     fak * S[1][22][9];
     S[10][22][10] = gma2*S[4][22][10]+     fak * (S[1][22][10] + 2.e0*S[4][22][4] );
     S[10][23][5] = gma2*S[4][23][5]+     fak * (S[1][23][5] + 4.e0*S[4][13][5] );
     S[10][23][6] = gma2*S[4][23][6]+     fak * (S[1][23][6] + 4.e0*S[4][13][6] + S[4][23][2] );
     S[10][23][7] = gma2*S[4][23][7]+     fak * (S[1][23][7] + 4.e0*S[4][13][7] + S[4][23][3] );
     S[10][23][8] = gma2*S[4][23][8]+     fak * (S[1][23][8] + 4.e0*S[4][13][8] );
     S[10][23][9] = gma2*S[4][23][9]+     fak * (S[1][23][9] + 4.e0*S[4][13][9] );
     S[10][23][10] = gma2*S[4][23][10]+     fak * (S[1][23][10] + 4.e0*S[4][13][10] + 2.e0*S[4][23][4] );
     S[10][24][5] = gma2*S[4][24][5]+     fak * S[1][24][5];
     S[10][24][6] = gma2*S[4][24][6]+     fak * (S[1][24][6] + S[4][24][2] );
     S[10][24][7] = gma2*S[4][24][7]+     fak * (S[1][24][7] + S[4][24][3] );
     S[10][24][8] = gma2*S[4][24][8]+     fak * S[1][24][8];
     S[10][24][9] = gma2*S[4][24][9]+     fak * S[1][24][9];
     S[10][24][10] = gma2*S[4][24][10]+     fak * (S[1][24][10] + 2.e0*S[4][24][4] );
     S[10][25][5] = gma2*S[4][25][5]+     fak * S[1][25][5];
     S[10][25][6] = gma2*S[4][25][6]+     fak * (S[1][25][6] + S[4][25][2] );
     S[10][25][7] = gma2*S[4][25][7]+     fak * (S[1][25][7] + S[4][25][3] );
     S[10][25][8] = gma2*S[4][25][8]+     fak * S[1][25][8];
     S[10][25][9] = gma2*S[4][25][9]+     fak * S[1][25][9];
     S[10][25][10] = gma2*S[4][25][10]+     fak * (S[1][25][10] + 2.e0*S[4][25][4] );
     S[10][26][5] = gma2*S[4][26][5]+     fak * (S[1][26][5] + S[4][11][5] );
     S[10][26][6] = gma2*S[4][26][6]+     fak * (S[1][26][6] + S[4][11][6] + S[4][26][2] );
     S[10][26][7] = gma2*S[4][26][7]+     fak * (S[1][26][7] + S[4][11][7] + S[4][26][3] );
     S[10][26][8] = gma2*S[4][26][8]+     fak * (S[1][26][8] + S[4][11][8] );
     S[10][26][9] = gma2*S[4][26][9]+     fak * (S[1][26][9] + S[4][11][9] );
     S[10][26][10] = gma2*S[4][26][10]+     fak * (S[1][26][10] + S[4][11][10] + 2.e0*S[4][26][4] );
     S[10][27][5] = gma2*S[4][27][5]+     fak * (S[1][27][5] + 3.e0*S[4][17][5] );
     S[10][27][6] = gma2*S[4][27][6]+     fak * (S[1][27][6] + 3.e0*S[4][17][6] + S[4][27][2] );
     S[10][27][7] = gma2*S[4][27][7]+     fak * (S[1][27][7] + 3.e0*S[4][17][7] + S[4][27][3] );
     S[10][27][8] = gma2*S[4][27][8]+     fak * (S[1][27][8] + 3.e0*S[4][17][8] );
     S[10][27][9] = gma2*S[4][27][9]+     fak * (S[1][27][9] + 3.e0*S[4][17][9] );
     S[10][27][10] = gma2*S[4][27][10]+     fak * (S[1][27][10] + 3.e0*S[4][17][10] + 2.e0*S[4][27][4] );
     S[10][28][5] = gma2*S[4][28][5]+     fak * (S[1][28][5] + S[4][12][5] );
     S[10][28][6] = gma2*S[4][28][6]+     fak * (S[1][28][6] + S[4][12][6] + S[4][28][2] );
     S[10][28][7] = gma2*S[4][28][7]+     fak * (S[1][28][7] + S[4][12][7] + S[4][28][3] );
     S[10][28][8] = gma2*S[4][28][8]+     fak * (S[1][28][8] + S[4][12][8] );
     S[10][28][9] = gma2*S[4][28][9]+     fak * (S[1][28][9] + S[4][12][9] );
     S[10][28][10] = gma2*S[4][28][10]+     fak * (S[1][28][10] + S[4][12][10] + 2.e0*S[4][28][4] );
     S[10][29][5] = gma2*S[4][29][5]+     fak * (S[1][29][5] + 3.e0*S[4][19][5] );
     S[10][29][6] = gma2*S[4][29][6]+     fak * (S[1][29][6] + 3.e0*S[4][19][6] + S[4][29][2] );
     S[10][29][7] = gma2*S[4][29][7]+     fak * (S[1][29][7] + 3.e0*S[4][19][7] + S[4][29][3] );
     S[10][29][8] = gma2*S[4][29][8]+     fak * (S[1][29][8] + 3.e0*S[4][19][8] );
     S[10][29][9] = gma2*S[4][29][9]+     fak * (S[1][29][9] + 3.e0*S[4][19][9] );
     S[10][29][10] = gma2*S[4][29][10]+     fak * (S[1][29][10] + 3.e0*S[4][19][10] + 2.e0*S[4][29][4] );
     S[10][30][5] = gma2*S[4][30][5]+     fak * (S[1][30][5] + 2.e0*S[4][16][5] );
     S[10][30][6] = gma2*S[4][30][6]+     fak * (S[1][30][6] + 2.e0*S[4][16][6] + S[4][30][2] );
     S[10][30][7] = gma2*S[4][30][7]+     fak * (S[1][30][7] + 2.e0*S[4][16][7] + S[4][30][3] );
     S[10][30][8] = gma2*S[4][30][8]+     fak * (S[1][30][8] + 2.e0*S[4][16][8] );
     S[10][30][9] = gma2*S[4][30][9]+     fak * (S[1][30][9] + 2.e0*S[4][16][9] );
     S[10][30][10] = gma2*S[4][30][10]+     fak * (S[1][30][10] + 2.e0*S[4][16][10] + 2.e0*S[4][30][4] );
     S[10][31][5] = gma2*S[4][31][5]+     fak * (S[1][31][5] + 2.e0*S[4][18][5] );
     S[10][31][6] = gma2*S[4][31][6]+     fak * (S[1][31][6] + 2.e0*S[4][18][6] + S[4][31][2] );
     S[10][31][7] = gma2*S[4][31][7]+     fak * (S[1][31][7] + 2.e0*S[4][18][7] + S[4][31][3] );
     S[10][31][8] = gma2*S[4][31][8]+     fak * (S[1][31][8] + 2.e0*S[4][18][8] );
     S[10][31][9] = gma2*S[4][31][9]+     fak * (S[1][31][9] + 2.e0*S[4][18][9] );
     S[10][31][10] = gma2*S[4][31][10]+     fak * (S[1][31][10] + 2.e0*S[4][18][10] + 2.e0*S[4][31][4] );
     S[10][32][5] = gma2*S[4][32][5]+     fak * S[1][32][5];
     S[10][32][6] = gma2*S[4][32][6]+     fak * (S[1][32][6] + S[4][32][2] );
     S[10][32][7] = gma2*S[4][32][7]+     fak * (S[1][32][7] + S[4][32][3] );
     S[10][32][8] = gma2*S[4][32][8]+     fak * S[1][32][8];
     S[10][32][9] = gma2*S[4][32][9]+     fak * S[1][32][9];
     S[10][32][10] = gma2*S[4][32][10]+     fak * (S[1][32][10] + 2.e0*S[4][32][4] );
     S[10][33][5] = gma2*S[4][33][5]+     fak * (S[1][33][5] + S[4][14][5] );
     S[10][33][6] = gma2*S[4][33][6]+     fak * (S[1][33][6] + S[4][14][6] + S[4][33][2] );
     S[10][33][7] = gma2*S[4][33][7]+     fak * (S[1][33][7] + S[4][14][7] + S[4][33][3] );
     S[10][33][8] = gma2*S[4][33][8]+     fak * (S[1][33][8] + S[4][14][8] );
     S[10][33][9] = gma2*S[4][33][9]+     fak * (S[1][33][9] + S[4][14][9] );
     S[10][33][10] = gma2*S[4][33][10]+     fak * (S[1][33][10] + S[4][14][10] + 2.e0*S[4][33][4] );
     S[10][34][5] = gma2*S[4][34][5]+     fak * (S[1][34][5] + S[4][15][5] );
     S[10][34][6] = gma2*S[4][34][6]+     fak * (S[1][34][6] + S[4][15][6] + S[4][34][2] );
     S[10][34][7] = gma2*S[4][34][7]+     fak * (S[1][34][7] + S[4][15][7] + S[4][34][3] );
     S[10][34][8] = gma2*S[4][34][8]+     fak * (S[1][34][8] + S[4][15][8] );
     S[10][34][9] = gma2*S[4][34][9]+     fak * (S[1][34][9] + S[4][15][9] );
     S[10][34][10] = gma2*S[4][34][10]+     fak * (S[1][34][10] + S[4][15][10] + 2.e0*S[4][34][4] );
     S[10][35][5] = gma2*S[4][35][5]+     fak * (S[1][35][5] + 2.e0*S[4][20][5] );
     S[10][35][6] = gma2*S[4][35][6]+     fak * (S[1][35][6] + 2.e0*S[4][20][6] + S[4][35][2] );
     S[10][35][7] = gma2*S[4][35][7]+     fak * (S[1][35][7] + 2.e0*S[4][20][7] + S[4][35][3] );
     S[10][35][8] = gma2*S[4][35][8]+     fak * (S[1][35][8] + 2.e0*S[4][20][8] );
     S[10][35][9] = gma2*S[4][35][9]+     fak * (S[1][35][9] + 2.e0*S[4][20][9] );
     S[10][35][10] = gma2*S[4][35][10]+     fak * (S[1][35][10] + 2.e0*S[4][20][10] + 2.e0*S[4][35][4] );
  }

            
            
            
            
           

        }

   

    }
}

